from ....Methods.Geometry.Circle import PointTranslateCircleError


def translate(self, Zt):
    """Translate the Circle object

    Parameters
    ----------
    self : Circle
        An Circle object

    Zt : complex
        Complex value for translation


    Returns
    -------
    None

    Raises
    -------
    PointTranslateCircleError
        The point must be a complex number or int or float

    """
    if (
        not isinstance(Zt, complex)
        and not isinstance(Zt, int)
        and not isinstance(Zt, float)
    ):
        raise PointTranslateCircleError(
            "The point must be a complex number or int or float"
        )

    # check if the Circle is correct
    self.check()

    # Modification from the translation of Circle
    if self.point_ref is not None:
        self.point_ref = self.point_ref + Zt
    self.center = self.center + Zt



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/DXFImport.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/DXFImport
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.DXFImport.get_surfaces import get_surfaces
except ImportError as error:
    get_surfaces = error


from ._check import InitUnKnowClassError


class DXFImport(FrozenClass):
    """Use a DXF to define a lamination"""

    VERSION = 1

    # cf Methods.Simulation.DXFImport.get_surfaces
    if isinstance(get_surfaces, ImportError):
        get_surfaces = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DXFImport method get_surfaces: " + str(get_surfaces)
                )
            )
        )
    else:
        get_surfaces = get_surfaces
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, file_path="", surf_dict=-1, BC_list=-1, init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "file_path" in list(init_dict.keys()):
                file_path = init_dict["file_path"]
            if "surf_dict" in list(init_dict.keys()):
                surf_dict = init_dict["surf_dict"]
            if "BC_list" in list(init_dict.keys()):
                BC_list = init_dict["BC_list"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.file_path = file_path
        self.surf_dict = surf_dict
        self.BC_list = BC_list

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        DXFImport_str = ""
        if self.parent is None:
            DXFImport_str += "parent = None " + linesep
        else:
            DXFImport_str += "parent = " + str(type(self.parent)) + " object" + linesep
        DXFImport_str += 'file_path = "' + str(self.file_path) + '"' + linesep
        DXFImport_str += "surf_dict = " + str(self.surf_dict) + linesep
        DXFImport_str += (
            "BC_list = "
            + linesep
            + str(self.BC_list).replace(linesep, linesep + "\t")
            + linesep
        )
        return DXFImport_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.file_path != self.file_path:
            return False
        if other.surf_dict != self.surf_dict:
            return False
        if other.BC_list != self.BC_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._file_path != self._file_path:
            diff_list.append(name + ".file_path")
        if other._surf_dict != self._surf_dict:
            diff_list.append(name + ".surf_dict")
        if other._BC_list != self._BC_list:
            diff_list.append(name + ".BC_list")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.file_path)
        if self.surf_dict is not None:
            for key, value in self.surf_dict.items():
                S += getsizeof(value) + getsizeof(key)
        if self.BC_list is not None:
            for value in self.BC_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        DXFImport_dict = dict()
        DXFImport_dict["file_path"] = self.file_path
        DXFImport_dict["surf_dict"] = (
            self.surf_dict.copy() if self.surf_dict is not None else None
        )
        DXFImport_dict["BC_list"] = (
            self.BC_list.copy() if self.BC_list is not None else None
        )
        # The class name is added to the dict for deserialisation purpose
        DXFImport_dict["__class__"] = "DXFImport"
        return DXFImport_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.file_path = None
        self.surf_dict = None
        self.BC_list = None

    def _get_file_path(self):
        """getter of file_path"""
        return self._file_path

    def _set_file_path(self, value):
        """setter of file_path"""
        check_var("file_path", value, "str")
        self._file_path = value

    file_path = property(
        fget=_get_file_path,
        fset=_set_file_path,
        doc=u"""Path to the DXF file to import

        :Type: str
        """,
    )

    def _get_surf_dict(self):
        """getter of surf_dict"""
        return self._surf_dict

    def _set_surf_dict(self, value):
        """setter of surf_dict"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("surf_dict", value, "dict")
        self._surf_dict = value

    surf_dict = property(
        fget=_get_surf_dict,
        fset=_set_surf_dict,
        doc=u"""Dictionnary to assign the surfaces: key=complex reference point coordinate, value=label of the surface

        :Type: dict
        """,
    )

    def _get_BC_list(self):
        """getter of BC_list"""
        return self._BC_list

    def _set_BC_list(self, value):
        """setter of BC_list"""
        if type(value) is int and value == -1:
            value = list()
        check_var("BC_list", value, "list")
        self._BC_list = value

    BC_list = property(
        fget=_get_BC_list,
        fset=_set_BC_list,
        doc=u"""List of tuple to apply boundary conditions (complex reference point coordinate, is_arc, label of the BC to apply)

        :Type: list
        """,
    )



# -*- coding: utf-8 -*-

from numpy import arcsin, exp


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW25
        A SlotW25 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """
    point_dict = self._comp_point_coordinate()
    Z4 = point_dict["Z4"]
    Rbo = self.get_Rbo()

    if self.is_outwards():
        return abs(Z4) - Rbo
    else:  # inward slot
        return Rbo - abs(Z4)



# -*- coding: utf-8 -*-

# File generated according to WMachineTable.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_WMachineTable(object):
    def setupUi(self, WMachineTable):
        if not WMachineTable.objectName():
            WMachineTable.setObjectName(u"WMachineTable")
        WMachineTable.resize(290, 357)
        WMachineTable.setMinimumSize(QSize(290, 0))
        WMachineTable.setMaximumSize(QSize(282, 16777215))
        self.verticalLayout = QVBoxLayout(WMachineTable)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.in_name = QLabel(WMachineTable)
        self.in_name.setObjectName(u"in_name")
        self.in_name.setAlignment(Qt.AlignCenter)

        self.verticalLayout.addWidget(self.in_name)

        self.tab_param = QTableWidget(WMachineTable)
        if self.tab_param.columnCount() < 2:
            self.tab_param.setColumnCount(2)
        __qtablewidgetitem = QTableWidgetItem()
        self.tab_param.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.tab_param.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        self.tab_param.setObjectName(u"tab_param")
        self.tab_param.setMinimumSize(QSize(270, 0))
        self.tab_param.setMaximumSize(QSize(260, 16777215))
        self.tab_param.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.tab_param.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)
        self.tab_param.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.tab_param.setAlternatingRowColors(True)
        self.tab_param.setColumnCount(2)
        self.tab_param.horizontalHeader().setCascadingSectionResizes(False)
        self.tab_param.horizontalHeader().setStretchLastSection(True)
        self.tab_param.verticalHeader().setVisible(False)
        self.tab_param.verticalHeader().setCascadingSectionResizes(False)

        self.verticalLayout.addWidget(self.tab_param)

        self.b_plot_machine = QPushButton(WMachineTable)
        self.b_plot_machine.setObjectName(u"b_plot_machine")

        self.verticalLayout.addWidget(self.b_plot_machine)

        self.b_mmf = QPushButton(WMachineTable)
        self.b_mmf.setObjectName(u"b_mmf")

        self.verticalLayout.addWidget(self.b_mmf)

        self.retranslateUi(WMachineTable)

        QMetaObject.connectSlotsByName(WMachineTable)

    # setupUi

    def retranslateUi(self, WMachineTable):
        WMachineTable.setWindowTitle(
            QCoreApplication.translate("WMachineTable", u"Form", None)
        )
        self.in_name.setText(
            QCoreApplication.translate(
                "WMachineTable", u"Main Machine Parameters", None
            )
        )
        ___qtablewidgetitem = self.tab_param.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(
            QCoreApplication.translate("WMachineTable", u"Name", None)
        )
        ___qtablewidgetitem1 = self.tab_param.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(
            QCoreApplication.translate("WMachineTable", u"Value", None)
        )
        self.b_plot_machine.setText(
            QCoreApplication.translate("WMachineTable", u"Plot Machine", None)
        )
        self.b_mmf.setText(
            QCoreApplication.translate("WMachineTable", u"Plot Stator Unit MMF", None)
        )

    # retranslateUi



def set_OP_from_array(self, OP_matrix, type_OP_matrix=1, index=0):
    """Extract the Operating Point from an OP_matrix

    Parameters
    ----------
    self : InputCurrent
        An InputCurrent object
    OP_matrix : ndarray
        Operating Point matrix (cf VarLoadCurrent)
    type_OP_matrix : int
        Select which kind of OP_matrix is used 0: (N0,I0,Phi0,T,P), 1:(N0,Id,Iq,T,P)
    index : int
        To select the line of the OP_matrix to use (default=0)
    """

    # Check OP_matrix
    assert len(OP_matrix.shape) == 2
    assert OP_matrix.shape[1] <= 5
    assert index < OP_matrix.shape[0]
    assert type_OP_matrix in [0, 1]

    self.N0 = OP_matrix[index, 0]
    if type_OP_matrix == 1:
        self.Id_ref = OP_matrix[index, 1]
        self.Iq_ref = OP_matrix[index, 2]
    else:
        self.set_Id_Iq(I0=OP_matrix[index, 1], Phi0=OP_matrix[index, 2])
    if OP_matrix.shape[1] > 3:
        self.Tem_av_ref = OP_matrix[index, 3]



# -*- coding: utf-8 -*-
import subprocess

from ....Functions.get_path_binary import get_path_binary


def gen_elmer_mesh(self, output):
    """Call ElmerGrid process to convert mesh from Gmsh format to Elmer's compatible

    Parameters
    ----------
    self : Magnetic
        a Magnetic object
    output : Output
        an Output object (to update)

    Returns
    -------
    success: {Boolean}
        Status flag
    """
    # ElmerGrid v8.4 must be installed and in the PATH
    project_name = self.get_path_save_fea(output)
    gmsh_filename = project_name + ".msh"
    elmermesh_folder = project_name
    ElmerGrid_binary = get_path_binary("ElmerGrid")
    cmd_elmergrid = [
        ElmerGrid_binary,
        "14",
        "2",
        gmsh_filename,
        "-2d",
        "-autoclean",
        "-names",
        "-out",
        elmermesh_folder,
    ]
    self.get_logger().info("Calling ElmerGrid: " + " ".join(map(str, cmd_elmergrid)))
    elmergrid = subprocess.Popen(
        cmd_elmergrid, stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )
    (stdout, stderr) = elmergrid.communicate()
    elmergrid.wait()
    self.get_logger().info(stdout.decode("UTF-8"))
    if elmergrid.returncode != 0:
        self.get_logger().info("ElmerGrid [Error]: " + stderr.decode("UTF-8"))
        return False
    elmergrid.terminate()
    self.get_logger().info("ElmerGrid call complete!")

    return True



# -*- coding: utf-8 -*-


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the machine

    Parameters
    ----------
    self : MachineUD
        MachineUD object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """
    surf_list = list()

    if self.frame is not None:
        surf_list.extend(self.frame.build_geometry(sym=sym, alpha=alpha, delta=delta))

    for lam in self.lam_list:
        surf_list.extend(lam.build_geometry(sym=sym, alpha=alpha, delta=delta))

    if self.shaft is not None:
        surf_list.extend(self.shaft.build_geometry(sym=sym, alpha=alpha, delta=delta))

    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/NotchEvenDist.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/NotchEvenDist
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Notch import Notch

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.NotchEvenDist.get_notch_list import get_notch_list
except ImportError as error:
    get_notch_list = error


from ._check import InitUnKnowClassError
from .Slot import Slot


class NotchEvenDist(Notch):
    """Class for evenly distributed notches (according to Zs)"""

    VERSION = 1

    # cf Methods.Machine.NotchEvenDist.get_notch_list
    if isinstance(get_notch_list, ImportError):
        get_notch_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use NotchEvenDist method get_notch_list: "
                    + str(get_notch_list)
                )
            )
        )
    else:
        get_notch_list = get_notch_list
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, alpha=0, notch_shape=-1, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "alpha" in list(init_dict.keys()):
                alpha = init_dict["alpha"]
            if "notch_shape" in list(init_dict.keys()):
                notch_shape = init_dict["notch_shape"]
        # Set the properties (value check and convertion are done in setter)
        self.alpha = alpha
        self.notch_shape = notch_shape
        # Call Notch init
        super(NotchEvenDist, self).__init__()
        # The class is frozen (in Notch init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        NotchEvenDist_str = ""
        # Get the properties inherited from Notch
        NotchEvenDist_str += super(NotchEvenDist, self).__str__()
        NotchEvenDist_str += "alpha = " + str(self.alpha) + linesep
        if self.notch_shape is not None:
            tmp = (
                self.notch_shape.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            )
            NotchEvenDist_str += "notch_shape = " + tmp
        else:
            NotchEvenDist_str += "notch_shape = None" + linesep + linesep
        return NotchEvenDist_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Notch
        if not super(NotchEvenDist, self).__eq__(other):
            return False
        if other.alpha != self.alpha:
            return False
        if other.notch_shape != self.notch_shape:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Notch
        diff_list.extend(super(NotchEvenDist, self).compare(other, name=name))
        if other._alpha != self._alpha:
            diff_list.append(name + ".alpha")
        if (other.notch_shape is None and self.notch_shape is not None) or (
            other.notch_shape is not None and self.notch_shape is None
        ):
            diff_list.append(name + ".notch_shape None mismatch")
        elif self.notch_shape is not None:
            diff_list.extend(
                self.notch_shape.compare(other.notch_shape, name=name + ".notch_shape")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Notch
        S += super(NotchEvenDist, self).__sizeof__()
        S += getsizeof(self.alpha)
        S += getsizeof(self.notch_shape)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Notch
        NotchEvenDist_dict = super(NotchEvenDist, self).as_dict(**kwargs)
        NotchEvenDist_dict["alpha"] = self.alpha
        if self.notch_shape is None:
            NotchEvenDist_dict["notch_shape"] = None
        else:
            NotchEvenDist_dict["notch_shape"] = self.notch_shape.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        NotchEvenDist_dict["__class__"] = "NotchEvenDist"
        return NotchEvenDist_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.alpha = None
        if self.notch_shape is not None:
            self.notch_shape._set_None()
        # Set to None the properties inherited from Notch
        super(NotchEvenDist, self)._set_None()

    def _get_alpha(self):
        """getter of alpha"""
        return self._alpha

    def _set_alpha(self, value):
        """setter of alpha"""
        check_var("alpha", value, "float")
        self._alpha = value

    alpha = property(
        fget=_get_alpha,
        fset=_set_alpha,
        doc=u"""angular positon of the first notch

        :Type: float
        """,
    )

    def _get_notch_shape(self):
        """getter of notch_shape"""
        return self._notch_shape

    def _set_notch_shape(self, value):
        """setter of notch_shape"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "notch_shape"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Slot()
        check_var("notch_shape", value, "Slot")
        self._notch_shape = value

        if self._notch_shape is not None:
            self._notch_shape.parent = self

    notch_shape = property(
        fget=_get_notch_shape,
        fset=_set_notch_shape,
        doc=u"""Shape of the Notch

        :Type: Slot
        """,
    )



from ....Classes.MeshSolution import MeshSolution
from ....Classes.MeshVTK import MeshVTK
from ....Classes.SolutionData import SolutionData
from ....Classes.SolutionVector import SolutionVector
from meshio import read
from SciDataTool import Data1D, VectorField, DataTime
from numpy import arange, append as np_append
from os.path import join


def get_meshsolution(self, output):
    """Build the MeshSolution objects from the FEA outputs.

    Parameters
    ----------
    self : MagElmer
        a MagElmer object
    output: Output
        An Output object

    Returns
    -------
    meshsol: MeshSolution
        a MeshSolution object with Elmer outputs at every time step
    """
    project_name = self.get_path_save_fea(output)
    elmermesh_folder = project_name
    meshsol = MeshSolution(label="Elmer MagnetoDynamics")
    if not self.is_get_mesh or not self.is_save_FEA:
        self.get_logger().info("MagElmer: MeshSolution is not stored by request.")
        return False

    meshvtk = MeshVTK(path=elmermesh_folder, name="step_t0002", format="vtu")
    meshsol.mesh = [meshvtk]

    result_filename = join(elmermesh_folder, "step_t0002.vtu")
    meshsolvtu = read(result_filename)
    # pt_data = meshsolvtu.point_data
    cell_data = meshsolvtu.cell_data

    # indices = arange(meshsolvtu.points.shape[0])
    indices = arange(
        meshsolvtu.cells[0].data.shape[0] + meshsolvtu.cells[1].data.shape[0]
    )

    Indices = Data1D(name="indice", values=indices, is_components=True)
    # store_dict = {
    #     "magnetic vector potential": {
    #         "name": "Magnetic Vector Potential A",
    #         "unit": "Wb",
    #         "symbol": "A",
    #         "norm": 1,
    #     },
    #     "magnetic flux density": {
    #         "name": "Magnetic Flux Density B",
    #         "unit": "T",
    #         "symbol": "B",
    #         "norm": 1,
    #     },
    #     "magnetic field strength": {
    #         "name": "Magnetic Field H",
    #         "unit": "A/m",
    #         "symbol": "H",
    #         "norm": 1,
    #     },
    #     "current density": {
    #         "name": "Current Density J",
    #         "unit": "A/mm2",
    #         "symbol": "J",
    #         "norm": 1,
    #     }
    # }
    store_dict = {
        "magnetic flux density e": {
            "name": "Magnetic Flux Density B",
            "unit": "T",
            "symbol": "B",
            "norm": 1,
        },
        "magnetic vector potential e": {
            "name": "Magnetic Vector Potential A",
            "unit": "Wb",
            "symbol": "A",
            "norm": 1,
        },
        "magnetic field strength e": {
            "name": "Magnetic Field H",
            "unit": "A/m",
            "symbol": "H",
            "norm": 1,
        },
        "current density e": {
            "name": "Current Density J",
            "unit": "A/mm2",
            "symbol": "J",
            "norm": 1,
        },
    }
    comp_ext = ["x", "y", "z"]
    sol_list = []
    # for key, value in pt_data.items():
    for key, value in cell_data.items():
        if key in store_dict.keys():
            # siz = value.shape[1]
            siz = value[0].shape[1]
            if siz > 3:
                print("Some Message")
                siz = 3
            components = []
            comp_name = []
            values = np_append(value[0], value[1], axis=0)
            for i in range(siz):
                if siz == 1:
                    ext = ""
                else:
                    ext = comp_ext[i]

                data = DataTime(
                    name=store_dict[key]["name"] + ext,
                    unit=store_dict[key]["unit"],
                    symbol=store_dict[key]["symbol"] + ext,
                    axes=[Indices],
                    # values=value[:, i],
                    values=values[:, i],
                    normalizations={"ref": store_dict[key]["norm"]},
                )

                components.append(data)
                comp_name.append("comp_" + ext)

            if siz == 1:
                field = components[0]
                sol_list.append(
                    SolutionData(
                        field=field,
                        # type_cell="point",
                        type_cell="triangle",
                        label=store_dict[key]["symbol"],
                    )
                )
            else:
                comps = {}
                for i in range(siz):
                    comps[comp_name[i]] = components[i]
                field = VectorField(
                    name=store_dict[key]["name"],
                    symbol=store_dict[key]["symbol"],
                    components=comps,
                )
                sol_list.append(
                    SolutionVector(
                        field=field,
                        # type_cell="point",
                        type_cell="triangle",
                        label=store_dict[key]["symbol"],
                    )
                )

    meshsol.solution = sol_list
    output.mag.meshsolution = meshsol

    return True



from numpy import pi


def comp_magnetization_dict(self, is_north=True):
    """Compute the dictionary of the magnetization direction of the magnets (key=magnet_X, value=angle[rad])
    Mangetization angle with Hole centered on Ox axis

    Parameters
    ----------
    self : Hole
        a Hole object
    is_north: True
        True: comp north magnetization, else add pi [rad]

    Returns
    -------
    mag_dict: dict
        magnetization dictionary (key=magnet_X, value=angle[rad])
    """

    if self.magnetization_dict_offset is not None:
        mag_dict = self.magnetization_dict_offset.copy()
        if not is_north:
            for key in mag_dict.keys():
                mag_dict[key] += pi
        return mag_dict
    else:
        return dict()



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotDC
        A SlotDC object

    Returns
    -------
    curve_list: list
        A list of 6 Segment and 5 Arc

    """

    [
        Z1,
        Z2,
        Z3,
        Z4,
        Z5,
        Z6,
        Z7,
        Z8,
        Z9,
        Z10,
        Z11,
        Z12,
        _,
        _,
        _,
    ] = self._comp_point_coordinate()

    # Adapt the lines according to the lamination
    if self.is_outwards():
        sign = 1
        direc = True
    else:
        sign = -1
        direc = False

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Arc1(Z2, Z3, sign * self.D1 / 2, is_trigo_direction=direc))
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Arc1(Z4, Z5, sign * self.D2 / 2, is_trigo_direction=direc))
    curve_list.append(Segment(Z5, Z6))

    curve_list.append(Arc1(Z6, Z7, sign * self.R3, is_trigo_direction=direc))

    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Arc1(Z8, Z9, sign * self.D2 / 2, is_trigo_direction=direc))
    curve_list.append(Segment(Z9, Z10))
    curve_list.append(Arc1(Z10, Z11, sign * self.D1 / 2, is_trigo_direction=direc))
    curve_list.append(Segment(Z11, Z12))

    return curve_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Geometry/SurfRing.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Geometry/SurfRing
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Surface import Surface

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Geometry.SurfRing.get_lines import get_lines
except ImportError as error:
    get_lines = error

try:
    from ..Methods.Geometry.SurfRing.rotate import rotate
except ImportError as error:
    rotate = error

try:
    from ..Methods.Geometry.SurfRing.translate import translate
except ImportError as error:
    translate = error

try:
    from ..Methods.Geometry.SurfRing.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Geometry.SurfRing.comp_length import comp_length
except ImportError as error:
    comp_length = error

try:
    from ..Methods.Geometry.SurfRing.get_patches import get_patches
except ImportError as error:
    get_patches = error

try:
    from ..Methods.Geometry.SurfRing.discretize import discretize
except ImportError as error:
    discretize = error

try:
    from ..Methods.Geometry.SurfRing.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Geometry.SurfRing.plot_lines import plot_lines
except ImportError as error:
    plot_lines = error

try:
    from ..Methods.Geometry.SurfRing.comp_point_ref import comp_point_ref
except ImportError as error:
    comp_point_ref = error


from ._check import InitUnKnowClassError
from .Surface import Surface


class SurfRing(Surface):
    """SurfRing is a surface between two closed surfaces (lamination surfaces for instance)"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Geometry.SurfRing.get_lines
    if isinstance(get_lines, ImportError):
        get_lines = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfRing method get_lines: " + str(get_lines))
            )
        )
    else:
        get_lines = get_lines
    # cf Methods.Geometry.SurfRing.rotate
    if isinstance(rotate, ImportError):
        rotate = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfRing method rotate: " + str(rotate))
            )
        )
    else:
        rotate = rotate
    # cf Methods.Geometry.SurfRing.translate
    if isinstance(translate, ImportError):
        translate = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfRing method translate: " + str(translate))
            )
        )
    else:
        translate = translate
    # cf Methods.Geometry.SurfRing.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfRing method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Geometry.SurfRing.comp_length
    if isinstance(comp_length, ImportError):
        comp_length = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfRing method comp_length: " + str(comp_length)
                )
            )
        )
    else:
        comp_length = comp_length
    # cf Methods.Geometry.SurfRing.get_patches
    if isinstance(get_patches, ImportError):
        get_patches = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfRing method get_patches: " + str(get_patches)
                )
            )
        )
    else:
        get_patches = get_patches
    # cf Methods.Geometry.SurfRing.discretize
    if isinstance(discretize, ImportError):
        discretize = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfRing method discretize: " + str(discretize))
            )
        )
    else:
        discretize = discretize
    # cf Methods.Geometry.SurfRing.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfRing method comp_surface: " + str(comp_surface)
                )
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Geometry.SurfRing.plot_lines
    if isinstance(plot_lines, ImportError):
        plot_lines = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfRing method plot_lines: " + str(plot_lines))
            )
        )
    else:
        plot_lines = plot_lines
    # cf Methods.Geometry.SurfRing.comp_point_ref
    if isinstance(comp_point_ref, ImportError):
        comp_point_ref = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfRing method comp_point_ref: " + str(comp_point_ref)
                )
            )
        )
    else:
        comp_point_ref = comp_point_ref
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        out_surf=-1,
        in_surf=-1,
        point_ref=0,
        label="",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "out_surf" in list(init_dict.keys()):
                out_surf = init_dict["out_surf"]
            if "in_surf" in list(init_dict.keys()):
                in_surf = init_dict["in_surf"]
            if "point_ref" in list(init_dict.keys()):
                point_ref = init_dict["point_ref"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
        # Set the properties (value check and convertion are done in setter)
        self.out_surf = out_surf
        self.in_surf = in_surf
        # Call Surface init
        super(SurfRing, self).__init__(point_ref=point_ref, label=label)
        # The class is frozen (in Surface init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SurfRing_str = ""
        # Get the properties inherited from Surface
        SurfRing_str += super(SurfRing, self).__str__()
        if self.out_surf is not None:
            tmp = self.out_surf.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            SurfRing_str += "out_surf = " + tmp
        else:
            SurfRing_str += "out_surf = None" + linesep + linesep
        if self.in_surf is not None:
            tmp = self.in_surf.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            SurfRing_str += "in_surf = " + tmp
        else:
            SurfRing_str += "in_surf = None" + linesep + linesep
        return SurfRing_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Surface
        if not super(SurfRing, self).__eq__(other):
            return False
        if other.out_surf != self.out_surf:
            return False
        if other.in_surf != self.in_surf:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Surface
        diff_list.extend(super(SurfRing, self).compare(other, name=name))
        if (other.out_surf is None and self.out_surf is not None) or (
            other.out_surf is not None and self.out_surf is None
        ):
            diff_list.append(name + ".out_surf None mismatch")
        elif self.out_surf is not None:
            diff_list.extend(
                self.out_surf.compare(other.out_surf, name=name + ".out_surf")
            )
        if (other.in_surf is None and self.in_surf is not None) or (
            other.in_surf is not None and self.in_surf is None
        ):
            diff_list.append(name + ".in_surf None mismatch")
        elif self.in_surf is not None:
            diff_list.extend(
                self.in_surf.compare(other.in_surf, name=name + ".in_surf")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Surface
        S += super(SurfRing, self).__sizeof__()
        S += getsizeof(self.out_surf)
        S += getsizeof(self.in_surf)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Surface
        SurfRing_dict = super(SurfRing, self).as_dict(**kwargs)
        if self.out_surf is None:
            SurfRing_dict["out_surf"] = None
        else:
            SurfRing_dict["out_surf"] = self.out_surf.as_dict(**kwargs)
        if self.in_surf is None:
            SurfRing_dict["in_surf"] = None
        else:
            SurfRing_dict["in_surf"] = self.in_surf.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SurfRing_dict["__class__"] = "SurfRing"
        return SurfRing_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.out_surf is not None:
            self.out_surf._set_None()
        if self.in_surf is not None:
            self.in_surf._set_None()
        # Set to None the properties inherited from Surface
        super(SurfRing, self)._set_None()

    def _get_out_surf(self):
        """getter of out_surf"""
        return self._out_surf

    def _set_out_surf(self, value):
        """setter of out_surf"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "out_surf"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Surface()
        check_var("out_surf", value, "Surface")
        self._out_surf = value

        if self._out_surf is not None:
            self._out_surf.parent = self

    out_surf = property(
        fget=_get_out_surf,
        fset=_set_out_surf,
        doc=u"""Outter surface

        :Type: Surface
        """,
    )

    def _get_in_surf(self):
        """getter of in_surf"""
        return self._in_surf

    def _set_in_surf(self, value):
        """setter of in_surf"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "in_surf"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Surface()
        check_var("in_surf", value, "Surface")
        self._in_surf = value

        if self._in_surf is not None:
            self._in_surf.parent = self

    in_surf = property(
        fget=_get_in_surf,
        fset=_set_in_surf,
        doc=u"""Inner surface

        :Type: Surface
        """,
    )



# -*- coding: utf-8 -*-


def get_m2_name(self):
    """Return the name of the current area unit

    Parameters
    ----------
    self : Unit
        A Unit object

    Returns
    -------
    unit_name : str
        Name of the current unit
    """
    if self.unit_m2 == 1:
        return "mm²"
    else:
        return "m²"



# -*- coding: utf-8 -*-


def is_synchronous(self):
    """Return if a machine is synchronous or not

    Parameters
    ----------
    self : MachineWRSM
        A MachineWRSM

    Returns
    -------
    is_synchronous: bool
        WRSM machines are synchronous

    """
    return True



# -*- coding: utf-8 -*-

# regions IDs and names (must be unique)

FEMM_GROUPS = dict()

# airgap GROUP for segments added between the rotor and stator (except middle airgap)
FEMM_GROUPS["GROUP_AG"] = {"name": "airgap", "ID": 0}
FEMM_GROUPS["GROUP_AGM"] = {"name": "airgap middle", "ID": 1}  # for torque calculations
# stator GROUPs
FEMM_GROUPS["GROUP_SC"] = {"name": "stator core", "ID": 10}
FEMM_GROUPS["GROUP_SW"] = {"name": "stator winding", "ID": 11}
FEMM_GROUPS["GROUP_SM"] = {"name": "stator magnets", "ID": 12}
FEMM_GROUPS["GROUP_SN"] = {"name": "stator notches", "ID": 13}
FEMM_GROUPS["GROUP_SV"] = {"name": "stator ventialtion duct", "ID": 14}
FEMM_GROUPS["GROUP_SSI"] = {"name": "stator slot isthmus", "ID": 15}
FEMM_GROUPS["GROUP_SH"] = {"name": "stator holes", "ID": 16}
# rotor GROUPs
FEMM_GROUPS["GROUP_RC"] = {"name": "rotor core", "ID": 20}
FEMM_GROUPS["GROUP_RW"] = {"name": "rotor winding", "ID": 21}
FEMM_GROUPS["GROUP_RM"] = {"name": "rotor magnets", "ID": 22}
FEMM_GROUPS["GROUP_RN"] = {"name": "rotor notches", "ID": 23}
FEMM_GROUPS["GROUP_RV"] = {"name": "rotor ventialtion duct", "ID": 24}
FEMM_GROUPS["GROUP_RSI"] = {"name": "rotor slot isthmus", "ID": 25}
FEMM_GROUPS["GROUP_RH"] = {"name": "rotor holes", "ID": 26}
# other GROUPs
FEMM_GROUPS["GROUP_IN"] = {"name": "inner shaft", "ID": 30}
FEMM_GROUPS["GROUP_FM"] = {"name": "fictitious magnets", "ID": 31}
FEMM_GROUPS["GROUP_FR"] = {"name": "frame", "ID": 32}

# global Values
pbtype = "planar"  # problem type
precision = 1e-8  # solver precision (default: 1e-8)
is_eddies = 0  # 1 to calculate eddy currents
hidebc = 0  # 1 to hide BC properties
type_yokeS = 0  # 0 for circular stator yoke, 1 for squared yoke
minangle = 15  # finite element mesh min angle
is_middleag = 0  # 1 to draw an arc in the middle of the airgap for torque calculations(
# not necessary)
acsolver = 0  # AC solver type 0 for successive approx, 1 for Newton

# dictionary to match FEMM boundary condition (value) with line labels (key)
# actual FEMM boundary condition is set in Arc and Segment draw_FEMM methods
boundary_prop = dict()
boundary_prop["airgap_line_1"] = "bc_ag1"
boundary_prop["sliding_line"] = "bc_ag2"
boundary_prop["airgap_line_2"] = "bc_ag3"
boundary_prop["Rotor_Yoke_Radius"] = "bc_A0"
boundary_prop["Stator_Yoke_Radius"] = "bc_A0"
boundary_prop["Rotor_Yoke_Side"] = "bc_r1"
boundary_prop["Stator_Yoke_Side"] = "bc_s1"



from ...Classes.SolutionData import SolutionData
from SciDataTool import DataTime
import numpy as np


def build_solution_data(field, axis_list, name="", symbol="", unit="", is_real=True):
    """Build the MeshSolution objets from FEMM outputs.

    Parameters
    ----------
    field : ndarray
        a data field
    axis_list : list
        a list of SciDataTool axis

    Returns
    -------
    solution: SolutionData
        a SolutionData object
    """

    data = DataTime(
        name=name,
        unit=unit,
        symbol=symbol,
        axes=axis_list,
        values=field,
        is_real=is_real,
    )

    return SolutionData(field=data, label=symbol)



from ....loggers import gen_logger_config_dict, DEFAULT_LOG_NAME
from logging.config import dictConfig
from os.path import join


def init_logger(self, output):
    """Create all the logger for the simulation

    Parameters
    ----------
    self : Simulation
        A Simulation object
    output : XOuput
        XOutput or Output object
    """

    if self.name in [None, ""]:
        log_name = DEFAULT_LOG_NAME
    else:
        log_name = self.name

    if self.layer == 0:  # Create logger only once for top simulation
        log_dict = gen_logger_config_dict(log_name)

        # Update log file
        log_path = join(output.get_path_result(), log_name + ".log")
        log_dict["handlers"]["file_handler"]["filename"] = log_path
        # Create the loggers
        dictConfig(log_dict)

    # Set the logger in the simulation
    self.logger_name = log_name
    self.machine.logger_name = log_name + ".Machine"
    if self.elec is not None:
        self.elec.logger_name = log_name + ".Electrical"
    if self.mag is not None:
        self.mag.logger_name = log_name + ".Magnetics"
    if self.force is not None:
        self.force.logger_name = log_name + ".Force"
    if self.struct is not None:
        self.struct.logger_name = log_name + ".Structural"
    # Set the logger in the Output
    output.logger_name = log_name
    output.elec.logger_name = log_name + ".Electrical"
    output.mag.logger_name = log_name + ".Magnetics"
    output.force.logger_name = log_name + ".Force"
    output.struct.logger_name = log_name + ".Structural"

    if self.layer == 0:
        # Display start of top simulation
        msg = "Starting running simulation"
        if self.name not in ["", None]:
            msg += " " + self.name
        if self.machine.name not in ["", None]:
            msg += " (machine=" + self.machine.name + ")"
        self.get_logger().debug("###################################")
        self.get_logger().info(msg)



# -*- coding: utf-8 -*-

import numpy as np


def get_point2cell(self, pt_indice):
    """Return all cell indices of cells containing a point.

    Parameters
    ----------
    self : MeshMat
        an MeshMat object
    pt_indice : int
        a point indice

    Returns
    -------
    pt_to_cell: ndarray
        Indices of cells containing the point

    """

    pt_to_cell = np.array([], dtype=int)

    for key in self.cell:
        connect = self.cell[key].connectivity
        indice_elem = self.cell[key].indice
        if len(connect[key].shape) > 1:  # If there is more than 1 element
            Ielem = np.where(connect[key] == pt_indice)[0]
            pt_to_cell = np.concatenate((pt_to_cell, indice_elem[key][Ielem]))
        else:
            if sum(connect[key] == pt_indice) > 0:
                pt_to_cell = np.concatenate((pt_to_cell, indice_elem[key]))

    return pt_to_cell



try:
    from deap.tools import cxOnePoint
except ImportError:
    cxOnePoint = None

import random


def cross(self, indiv1, indiv2):
    """Perform the crossover (One crossover by default)

    Parameters
    ----------
    self : OptiGenNsga2Deap
        Optimization solver

    indiv1 : individual
        first individual to modify
    indiv2 : individual
        second individual to modify

    Returns
    -------
    is_cross : bool
        True if the crossover append
    """

    if random.random() < self.p_cross:
        if self.crossover == None:
            if cxOnePoint == None:
                raise ImportError("deap module is needed.")
            else:
                cxOnePoint(indiv1, indiv2)
        else:
            self.crossover(indiv1, indiv2)

        return True
    else:
        return False



# -*- coding: utf-8 -*-

from numpy import cos


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW29
        A SlotW29 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """
    Rbo = self.get_Rbo()

    # Computation of the arc height
    alpha = self.comp_angle_opening() / 2
    Harc = float(Rbo * (1 - cos(alpha)))

    if self.is_outwards():
        return abs(Rbo - Harc + self.H0 + self.H1 + self.H2 + 1j * self.W2 / 2.0) - Rbo
    else:
        return self.H0 + self.H1 + self.H2 + Harc



# -*- coding: utf-8 -*-
from SciDataTool import DataFreq, VectorField, Data1D
from numpy import power, where, repeat, newaxis, multiply


def comp_AGSF_transfer(self, output, rnoise=None):
    """Method to compute Air-Gap Surface Force transfer from middle air-gap
    radius to stator bore radius.

    From publication:
        PILE, Raphaël, LE BESNERAIS, Jean, PARENT, Guillaume, et al. Analytical
        study of air-gap surface force–application to electrical machines. Open
        Physics, 2020, vol. 18, no 1, p. 658-673.
        https://www.degruyter.com/view/journals/phys/18/1/article-p658.xml

    Parameters
    ----------
    self: Force
        a Force object
    output : Output
        an Output object

    """

    # Inputs
    Rag = output.force.Rag

    if self.Rsbo_enforced_transfer is None:
        Rsbo = output.simu.machine.stator.Rint
    else:
        Rsbo = self.Rsbo_enforced_transfer

    AGSF = output.force.AGSF

    arg_list = ["freqs", "wavenumber"]
    result_freq = AGSF.get_rphiz_along(*arg_list)
    Prad_wr = result_freq["radial"]
    Ptan_wr = result_freq["tangential"]
    wavenumber = result_freq["wavenumber"]
    freqs = result_freq["freqs"]
    Nf = len(freqs)

    Ratio = Rag / Rsbo

    # Transfer coefficients Eq. (46)
    Sn = (Ratio ** 2) * (power(Ratio, wavenumber) + power(Ratio, -wavenumber)) / 2
    Cn = (Ratio ** 2) * (power(Ratio, wavenumber) - power(Ratio, -wavenumber)) / 2

    # Noise filtering (useful with magnetic FEA)
    if rnoise is not None:
        Inoise = where(abs(wavenumber) > rnoise)[0]
        Sn[Inoise] = 1
        Cn[Inoise] = 0

    XSn = repeat(Sn[..., newaxis], Nf, axis=1).transpose()
    XCn = repeat(Cn[..., newaxis], Nf, axis=1).transpose()

    # Transfer law Eq. (45)
    Prad_wr_TR = multiply(XSn, Prad_wr) + 1j * multiply(XCn, Ptan_wr)
    Ptan_wr_TR = multiply(XSn, Ptan_wr) - 1j * multiply(XCn, Prad_wr)

    # Save results as Data objects
    Datafreqs = Data1D(name="freqs", values=freqs)
    Datawavenumbers = Data1D(name="wavenumber", values=wavenumber)

    axes_list = [Datafreqs, Datawavenumbers]

    AGSF_TR = VectorField(
        name="Air gap Surface Force",
        symbol="AGSF",
    )

    AGSF_TR.components["radial"] = DataFreq(
        name="Radial AGSF",
        unit="N/m²",
        symbol="AGSF_r",
        axes=axes_list,
        values=Prad_wr_TR,
    )

    AGSF_TR.components["tangential"] = DataFreq(
        name="Tangential AGSF",
        unit="N/m²",
        symbol="AGSF_t",
        axes=axes_list,
        values=Ptan_wr_TR,
    )

    # Replace original AGSF
    output.force.AGSF = AGSF_TR
    output.force.Rag = Rsbo



# -*- coding: utf-8 -*-

from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from ......Classes.CondType21 import CondType21
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SBar.PCondType21.Gen_PCondType21 import (
    Gen_PCondType21,
)


class PCondType21(Gen_PCondType21, QWidget):
    """Page to setup Conductor Type 21"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for SBar combobox
    cond_name = "Rectangular bar"
    cond_type = CondType21

    def __init__(self, machine=None, matlib=None):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : PCondType21
            A PCondType21 widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Setup material combobox according to matlib names
        self.matlib = matlib
        self.w_mat.def_mat = "Copper1"

        # Set FloatEdit unit
        self.lf_Hbar.unit = "m"
        self.lf_Wbar.unit = "m"

        # Set unit name (m ou mm)
        self.u = gui_option.unit
        wid_list = [self.unit_Hbar, self.unit_Wbar]
        for wid in wid_list:
            wid.setText(self.u.get_m_name())

        # Fill the fields with the machine values (if they're filled)
        self.machine = machine

        conductor = machine.rotor.winding.conductor
        # Make sure that the rotor's conductor is a 2_1
        if conductor is None or not isinstance(conductor, CondType21):
            self.machine.rotor.winding.conductor = CondType21()
            self.machine.rotor.winding.conductor._set_None()
            # Make sure to re-set conductor with the new object
            conductor = machine.rotor.winding.conductor

        self.lf_Hbar.setValue(conductor.Hbar)
        self.lf_Wbar.setValue(conductor.Wbar)

        self.w_mat.update(conductor, "cond_mat", self.matlib)

        # Display the main output
        self.w_out.comp_output()

        # Connect the widget
        self.lf_Hbar.editingFinished.connect(self.set_Hbar)
        self.lf_Wbar.editingFinished.connect(self.set_Wbar)
        self.w_mat.saveNeeded.connect(self.emit_save)

    def emit_save(self):
        """Emit the saveNeeded signal"""
        self.saveNeeded.emit()

    def set_Hbar(self):
        """Signal to update the value of Hbar according to the line edit

        Parameters
        ----------
        self : PCondType21
            A PCondType21 object

        Returns
        -------

        """
        self.machine.rotor.winding.conductor.Hbar = self.lf_Hbar.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Wbar(self):
        """Signal to update the value of Wbar according to the line edit

        Parameters
        ----------
        self : PCondType21
            A PCondType21 object

        Returns
        -------

        """
        self.machine.rotor.winding.conductor.Wbar = self.lf_Wbar.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()



# -*- coding: utf-8 -*-

from numpy import linspace, zeros

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine


def get_surface_active(self, alpha=0, delta=0):
    """Return the full active surface

    Parameters
    ----------
    self : SlotM17
        A SlotM17 object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_act: Surface
        Surface corresponding to the Active Area
    """

    self.check()
    # get the name of the lamination
    st = self.get_name_lam()
    Rext = self.parent.Rext

    Z1 = -Rext * 1j
    Z2 = Rext
    Z3 = Rext * 1j
    curve_list = list()
    curve_list.append(Arc1(Z1, Z2, Rext, True))
    curve_list.append(Arc1(Z2, Z3, Rext, True))
    curve_list.append(Segment(Z3, Z1))

    surface = SurfLine(
        line_list=curve_list, label="Wind_" + st + "_R0_T0_S0", point_ref=Rext / 2
    )

    # Apply transformation
    surface.rotate(alpha)
    surface.translate(delta)

    return surface



# -*- coding: utf-8 -*-

# File generated according to DMatLib.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_DMatLib(object):
    def setupUi(self, DMatLib):
        if not DMatLib.objectName():
            DMatLib.setObjectName(u"DMatLib")
        DMatLib.resize(746, 536)
        self.verticalLayout_11 = QVBoxLayout(DMatLib)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.le_search = QLineEdit(DMatLib)
        self.le_search.setObjectName(u"le_search")

        self.horizontalLayout_9.addWidget(self.le_search)

        self.img_search = QLabel(DMatLib)
        self.img_search.setObjectName(u"img_search")
        self.img_search.setPixmap(
            QPixmap(
                u"\n"
                "                                                    :/images/images/icon/search.png\n"
                "                                                "
            )
        )
        self.img_search.setScaledContents(True)

        self.horizontalLayout_9.addWidget(self.img_search)

        self.verticalLayout_3.addLayout(self.horizontalLayout_9)

        self.verticalLayout_12 = QVBoxLayout()
        self.verticalLayout_12.setObjectName(u"verticalLayout_12")
        self.in_reference_mat_lib = QLabel(DMatLib)
        self.in_reference_mat_lib.setObjectName(u"in_reference_mat_lib")

        self.verticalLayout_12.addWidget(self.in_reference_mat_lib)

        self.nav_mat = QListWidget(DMatLib)
        self.nav_mat.setObjectName(u"nav_mat")

        self.verticalLayout_12.addWidget(self.nav_mat)

        self.in_machine_mat = QLabel(DMatLib)
        self.in_machine_mat.setObjectName(u"in_machine_mat")

        self.verticalLayout_12.addWidget(self.in_machine_mat)

        self.nav_mat_mach = QListWidget(DMatLib)
        self.nav_mat_mach.setObjectName(u"nav_mat_mach")

        self.verticalLayout_12.addWidget(self.nav_mat_mach)

        self.verticalLayout_3.addLayout(self.verticalLayout_12)

        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.horizontalLayout_8.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.b_edit = QPushButton(DMatLib)
        self.b_edit.setObjectName(u"b_edit")

        self.horizontalLayout_8.addWidget(self.b_edit)

        self.b_duplicate = QPushButton(DMatLib)
        self.b_duplicate.setObjectName(u"b_duplicate")

        self.horizontalLayout_8.addWidget(self.b_duplicate)

        self.b_delete = QPushButton(DMatLib)
        self.b_delete.setObjectName(u"b_delete")

        self.horizontalLayout_8.addWidget(self.b_delete)

        self.verticalLayout_3.addLayout(self.horizontalLayout_8)

        self.horizontalLayout_2.addLayout(self.verticalLayout_3)

        self.verticalLayout_10 = QVBoxLayout()
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.out_name = QLabel(DMatLib)
        self.out_name.setObjectName(u"out_name")

        self.verticalLayout_10.addWidget(self.out_name)

        self.out_iso = QLabel(DMatLib)
        self.out_iso.setObjectName(u"out_iso")

        self.verticalLayout_10.addWidget(self.out_iso)

        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.g_elec = QGroupBox(DMatLib)
        self.g_elec.setObjectName(u"g_elec")
        self.verticalLayout_4 = QVBoxLayout(self.g_elec)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.out_rho_elec = QLabel(self.g_elec)
        self.out_rho_elec.setObjectName(u"out_rho_elec")

        self.verticalLayout_4.addWidget(self.out_rho_elec)

        self.out_epsr = QLabel(self.g_elec)
        self.out_epsr.setObjectName(u"out_epsr")

        self.verticalLayout_4.addWidget(self.out_epsr)

        self.gridLayout.addWidget(self.g_elec, 0, 0, 1, 1)

        self.g_eco = QGroupBox(DMatLib)
        self.g_eco.setObjectName(u"g_eco")
        self.verticalLayout_5 = QVBoxLayout(self.g_eco)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.out_cost_unit = QLabel(self.g_eco)
        self.out_cost_unit.setObjectName(u"out_cost_unit")

        self.verticalLayout_5.addWidget(self.out_cost_unit)

        self.gridLayout.addWidget(self.g_eco, 0, 1, 1, 1)

        self.g_thermics = QGroupBox(DMatLib)
        self.g_thermics.setObjectName(u"g_thermics")
        self.verticalLayout_6 = QVBoxLayout(self.g_thermics)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.out_Cp = QLabel(self.g_thermics)
        self.out_Cp.setObjectName(u"out_Cp")

        self.verticalLayout_6.addWidget(self.out_Cp)

        self.out_alpha = QLabel(self.g_thermics)
        self.out_alpha.setObjectName(u"out_alpha")

        self.verticalLayout_6.addWidget(self.out_alpha)

        self.nav_iso_therm = QStackedWidget(self.g_thermics)
        self.nav_iso_therm.setObjectName(u"nav_iso_therm")
        self.page_iso_therm = QWidget()
        self.page_iso_therm.setObjectName(u"page_iso_therm")
        self.verticalLayout = QVBoxLayout(self.page_iso_therm)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.out_L = QLabel(self.page_iso_therm)
        self.out_L.setObjectName(u"out_L")

        self.verticalLayout.addWidget(self.out_L)

        self.nav_iso_therm.addWidget(self.page_iso_therm)
        self.page_niso_therm = QWidget()
        self.page_niso_therm.setObjectName(u"page_niso_therm")
        self.verticalLayout_2 = QVBoxLayout(self.page_niso_therm)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.out_LX = QLabel(self.page_niso_therm)
        self.out_LX.setObjectName(u"out_LX")

        self.verticalLayout_2.addWidget(self.out_LX)

        self.out_LY = QLabel(self.page_niso_therm)
        self.out_LY.setObjectName(u"out_LY")

        self.verticalLayout_2.addWidget(self.out_LY)

        self.out_LZ = QLabel(self.page_niso_therm)
        self.out_LZ.setObjectName(u"out_LZ")

        self.verticalLayout_2.addWidget(self.out_LZ)

        self.nav_iso_therm.addWidget(self.page_niso_therm)

        self.verticalLayout_6.addWidget(self.nav_iso_therm)

        self.gridLayout.addWidget(self.g_thermics, 1, 0, 1, 1)

        self.g_mechanics = QGroupBox(DMatLib)
        self.g_mechanics.setObjectName(u"g_mechanics")
        self.verticalLayout_9 = QVBoxLayout(self.g_mechanics)
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.out_rho_meca = QLabel(self.g_mechanics)
        self.out_rho_meca.setObjectName(u"out_rho_meca")

        self.verticalLayout_9.addWidget(self.out_rho_meca)

        self.nav_iso_meca = QStackedWidget(self.g_mechanics)
        self.nav_iso_meca.setObjectName(u"nav_iso_meca")
        self.page_iso_meca = QWidget()
        self.page_iso_meca.setObjectName(u"page_iso_meca")
        self.verticalLayout_7 = QVBoxLayout(self.page_iso_meca)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.out_E = QLabel(self.page_iso_meca)
        self.out_E.setObjectName(u"out_E")

        self.verticalLayout_7.addWidget(self.out_E)

        self.out_nu = QLabel(self.page_iso_meca)
        self.out_nu.setObjectName(u"out_nu")

        self.verticalLayout_7.addWidget(self.out_nu)

        self.out_G = QLabel(self.page_iso_meca)
        self.out_G.setObjectName(u"out_G")

        self.verticalLayout_7.addWidget(self.out_G)

        self.nav_iso_meca.addWidget(self.page_iso_meca)
        self.page_nios_meca = QWidget()
        self.page_nios_meca.setObjectName(u"page_nios_meca")
        self.gridLayout_2 = QGridLayout(self.page_nios_meca)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.in_E = QLabel(self.page_nios_meca)
        self.in_E.setObjectName(u"in_E")

        self.gridLayout_2.addWidget(self.in_E, 1, 0, 1, 1)

        self.out_EX = QLabel(self.page_nios_meca)
        self.out_EX.setObjectName(u"out_EX")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.out_EX.sizePolicy().hasHeightForWidth())
        self.out_EX.setSizePolicy(sizePolicy)
        self.out_EX.setMinimumSize(QSize(50, 20))
        self.out_EX.setMaximumSize(QSize(50, 20))
        self.out_EX.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_EX, 1, 1, 1, 1)

        self.out_EY = QLabel(self.page_nios_meca)
        self.out_EY.setObjectName(u"out_EY")
        sizePolicy.setHeightForWidth(self.out_EY.sizePolicy().hasHeightForWidth())
        self.out_EY.setSizePolicy(sizePolicy)
        self.out_EY.setMinimumSize(QSize(50, 20))
        self.out_EY.setMaximumSize(QSize(50, 20))
        self.out_EY.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_EY, 1, 2, 1, 1)

        self.out_EZ = QLabel(self.page_nios_meca)
        self.out_EZ.setObjectName(u"out_EZ")
        sizePolicy.setHeightForWidth(self.out_EZ.sizePolicy().hasHeightForWidth())
        self.out_EZ.setSizePolicy(sizePolicy)
        self.out_EZ.setMinimumSize(QSize(50, 20))
        self.out_EZ.setMaximumSize(QSize(50, 20))
        self.out_EZ.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_EZ, 1, 3, 1, 1)

        self.in_nu = QLabel(self.page_nios_meca)
        self.in_nu.setObjectName(u"in_nu")

        self.gridLayout_2.addWidget(self.in_nu, 3, 0, 1, 1)

        self.out_nu_XY = QLabel(self.page_nios_meca)
        self.out_nu_XY.setObjectName(u"out_nu_XY")
        sizePolicy.setHeightForWidth(self.out_nu_XY.sizePolicy().hasHeightForWidth())
        self.out_nu_XY.setSizePolicy(sizePolicy)
        self.out_nu_XY.setMinimumSize(QSize(50, 20))
        self.out_nu_XY.setMaximumSize(QSize(50, 20))
        self.out_nu_XY.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_nu_XY, 3, 1, 1, 1)

        self.out_nu_XZ = QLabel(self.page_nios_meca)
        self.out_nu_XZ.setObjectName(u"out_nu_XZ")
        sizePolicy.setHeightForWidth(self.out_nu_XZ.sizePolicy().hasHeightForWidth())
        self.out_nu_XZ.setSizePolicy(sizePolicy)
        self.out_nu_XZ.setMinimumSize(QSize(50, 20))
        self.out_nu_XZ.setMaximumSize(QSize(50, 20))
        self.out_nu_XZ.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_nu_XZ, 3, 2, 1, 1)

        self.out_nu_YZ = QLabel(self.page_nios_meca)
        self.out_nu_YZ.setObjectName(u"out_nu_YZ")
        sizePolicy.setHeightForWidth(self.out_nu_YZ.sizePolicy().hasHeightForWidth())
        self.out_nu_YZ.setSizePolicy(sizePolicy)
        self.out_nu_YZ.setMinimumSize(QSize(50, 20))
        self.out_nu_YZ.setMaximumSize(QSize(50, 20))
        self.out_nu_YZ.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_nu_YZ, 3, 3, 1, 1)

        self.in_G = QLabel(self.page_nios_meca)
        self.in_G.setObjectName(u"in_G")

        self.gridLayout_2.addWidget(self.in_G, 4, 0, 1, 1)

        self.out_GXY = QLabel(self.page_nios_meca)
        self.out_GXY.setObjectName(u"out_GXY")
        sizePolicy.setHeightForWidth(self.out_GXY.sizePolicy().hasHeightForWidth())
        self.out_GXY.setSizePolicy(sizePolicy)
        self.out_GXY.setMinimumSize(QSize(50, 20))
        self.out_GXY.setMaximumSize(QSize(50, 20))
        self.out_GXY.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_GXY, 4, 1, 1, 1)

        self.out_GXZ = QLabel(self.page_nios_meca)
        self.out_GXZ.setObjectName(u"out_GXZ")
        sizePolicy.setHeightForWidth(self.out_GXZ.sizePolicy().hasHeightForWidth())
        self.out_GXZ.setSizePolicy(sizePolicy)
        self.out_GXZ.setMinimumSize(QSize(50, 20))
        self.out_GXZ.setMaximumSize(QSize(50, 20))
        self.out_GXZ.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_GXZ, 4, 2, 1, 1)

        self.out_GYZ = QLabel(self.page_nios_meca)
        self.out_GYZ.setObjectName(u"out_GYZ")
        sizePolicy.setHeightForWidth(self.out_GYZ.sizePolicy().hasHeightForWidth())
        self.out_GYZ.setSizePolicy(sizePolicy)
        self.out_GYZ.setMinimumSize(QSize(50, 20))
        self.out_GYZ.setMaximumSize(QSize(50, 20))
        self.out_GYZ.setFrameShape(QFrame.NoFrame)

        self.gridLayout_2.addWidget(self.out_GYZ, 4, 3, 1, 1)

        self.in_XY = QLabel(self.page_nios_meca)
        self.in_XY.setObjectName(u"in_XY")

        self.gridLayout_2.addWidget(self.in_XY, 2, 1, 1, 1)

        self.in_XZ = QLabel(self.page_nios_meca)
        self.in_XZ.setObjectName(u"in_XZ")

        self.gridLayout_2.addWidget(self.in_XZ, 2, 2, 1, 1)

        self.in_Y = QLabel(self.page_nios_meca)
        self.in_Y.setObjectName(u"in_Y")

        self.gridLayout_2.addWidget(self.in_Y, 0, 2, 1, 1)

        self.in_X = QLabel(self.page_nios_meca)
        self.in_X.setObjectName(u"in_X")

        self.gridLayout_2.addWidget(self.in_X, 0, 1, 1, 1)

        self.in_YZ = QLabel(self.page_nios_meca)
        self.in_YZ.setObjectName(u"in_YZ")

        self.gridLayout_2.addWidget(self.in_YZ, 2, 3, 1, 1)

        self.in_Z = QLabel(self.page_nios_meca)
        self.in_Z.setObjectName(u"in_Z")

        self.gridLayout_2.addWidget(self.in_Z, 0, 3, 1, 1)

        self.nav_iso_meca.addWidget(self.page_nios_meca)

        self.verticalLayout_9.addWidget(self.nav_iso_meca)

        self.gridLayout.addWidget(self.g_mechanics, 1, 1, 1, 1)

        self.verticalLayout_10.addLayout(self.gridLayout)

        self.g_mag = QGroupBox(DMatLib)
        self.g_mag.setObjectName(u"g_mag")
        self.verticalLayout_8 = QVBoxLayout(self.g_mag)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.out_mur_lin = QLabel(self.g_mag)
        self.out_mur_lin.setObjectName(u"out_mur_lin")

        self.verticalLayout_8.addWidget(self.out_mur_lin)

        self.out_Brm20 = QLabel(self.g_mag)
        self.out_Brm20.setObjectName(u"out_Brm20")

        self.verticalLayout_8.addWidget(self.out_Brm20)

        self.out_alpha_Br = QLabel(self.g_mag)
        self.out_alpha_Br.setObjectName(u"out_alpha_Br")

        self.verticalLayout_8.addWidget(self.out_alpha_Br)

        self.out_wlam = QLabel(self.g_mag)
        self.out_wlam.setObjectName(u"out_wlam")

        self.verticalLayout_8.addWidget(self.out_wlam)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_BH_curve = QLabel(self.g_mag)
        self.in_BH_curve.setObjectName(u"in_BH_curve")

        self.horizontalLayout.addWidget(self.in_BH_curve)

        self.out_BH = QLabel(self.g_mag)
        self.out_BH.setObjectName(u"out_BH")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.out_BH.sizePolicy().hasHeightForWidth())
        self.out_BH.setSizePolicy(sizePolicy1)
        self.out_BH.setAlignment(Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter)

        self.horizontalLayout.addWidget(self.out_BH)

        self.verticalLayout_8.addLayout(self.horizontalLayout)

        self.verticalLayout_10.addWidget(self.g_mag)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_10.addItem(self.verticalSpacer)

        self.horizontalLayout_2.addLayout(self.verticalLayout_10)

        self.verticalLayout_11.addLayout(self.horizontalLayout_2)

        self.b_close = QDialogButtonBox(DMatLib)
        self.b_close.setObjectName(u"b_close")
        self.b_close.setOrientation(Qt.Horizontal)
        self.b_close.setStandardButtons(QDialogButtonBox.NoButton)

        self.verticalLayout_11.addWidget(self.b_close)

        self.b_close.raise_()

        self.retranslateUi(DMatLib)
        self.b_close.accepted.connect(DMatLib.accept)
        self.b_close.rejected.connect(DMatLib.reject)

        self.nav_iso_therm.setCurrentIndex(1)
        self.nav_iso_meca.setCurrentIndex(0)

        QMetaObject.connectSlotsByName(DMatLib)

    # setupUi

    def retranslateUi(self, DMatLib):
        DMatLib.setWindowTitle(
            QCoreApplication.translate("DMatLib", u"Material Library", None)
        )
        self.img_search.setText("")
        self.in_reference_mat_lib.setText(
            QCoreApplication.translate("DMatLib", u"Material library", None)
        )
        self.in_machine_mat.setText(
            QCoreApplication.translate("DMatLib", u"Machine materials", None)
        )
        self.b_edit.setText(QCoreApplication.translate("DMatLib", u"Edit", None))
        self.b_duplicate.setText(
            QCoreApplication.translate("DMatLib", u"New from", None)
        )
        self.b_delete.setText(QCoreApplication.translate("DMatLib", u"Delete", None))
        self.out_name.setText(
            QCoreApplication.translate("DMatLib", u"name : M400-50A", None)
        )
        self.out_iso.setText(QCoreApplication.translate("DMatLib", u"isotropic", None))
        self.g_elec.setTitle(QCoreApplication.translate("DMatLib", u"Electrical", None))
        self.out_rho_elec.setText(
            QCoreApplication.translate("DMatLib", u"rho = 10e-3 ohm.m", None)
        )
        self.out_epsr.setText(
            QCoreApplication.translate("DMatLib", u"epsr = 0.5", None)
        )
        self.g_eco.setTitle(QCoreApplication.translate("DMatLib", u"Economical", None))
        self.out_cost_unit.setText(
            QCoreApplication.translate(
                "DMatLib", u"cost_unit = 10.5 \u20ac/kg          ", None
            )
        )
        self.g_thermics.setTitle(
            QCoreApplication.translate("DMatLib", u"Thermics", None)
        )
        self.out_Cp.setText(
            QCoreApplication.translate("DMatLib", u"Cp = 0.52 W/kg/K            ", None)
        )
        self.out_alpha.setText(
            QCoreApplication.translate("DMatLib", u"alpha = 0.52", None)
        )
        self.out_L.setText(
            QCoreApplication.translate("DMatLib", u"Lambda = 5200 W/K", None)
        )
        self.out_LX.setText(
            QCoreApplication.translate("DMatLib", u"Lambda X = 5200 W/K", None)
        )
        self.out_LY.setText(
            QCoreApplication.translate("DMatLib", u"Lambda Y = 5200 W/K", None)
        )
        self.out_LZ.setText(
            QCoreApplication.translate("DMatLib", u"Lambda Z = 5200 W/K", None)
        )
        self.g_mechanics.setTitle(
            QCoreApplication.translate("DMatLib", u"Mechanics", None)
        )
        self.out_rho_meca.setText(
            QCoreApplication.translate("DMatLib", u"rho = 7500 kg/m^3 ", None)
        )
        self.out_E.setText(QCoreApplication.translate("DMatLib", u"E = 2360 Pa", None))
        self.out_nu.setText(QCoreApplication.translate("DMatLib", u"nu = 0.5", None))
        self.out_G.setText(
            QCoreApplication.translate("DMatLib", u"G = 10.000 Pa", None)
        )
        self.in_E.setText(QCoreApplication.translate("DMatLib", u"E", None))
        self.out_EX.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                1.23e+10\n"
                "                                                                            ",
                None,
            )
        )
        self.out_EY.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                10\n"
                "                                                                            ",
                None,
            )
        )
        self.out_EZ.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                55.3\n"
                "                                                                            ",
                None,
            )
        )
        self.in_nu.setText(QCoreApplication.translate("DMatLib", u"nu", None))
        self.out_nu_XY.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                0.5\n"
                "                                                                            ",
                None,
            )
        )
        self.out_nu_XZ.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                0.5\n"
                "                                                                            ",
                None,
            )
        )
        self.out_nu_YZ.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                0.6\n"
                "                                                                            ",
                None,
            )
        )
        self.in_G.setText(QCoreApplication.translate("DMatLib", u"G", None))
        self.out_GXY.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                1023\n"
                "                                                                            ",
                None,
            )
        )
        self.out_GXZ.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                1024\n"
                "                                                                            ",
                None,
            )
        )
        self.out_GYZ.setText(
            QCoreApplication.translate(
                "DMatLib",
                u"\n"
                "                                                                                1024\n"
                "                                                                            ",
                None,
            )
        )
        self.in_XY.setText(QCoreApplication.translate("DMatLib", u"XY", None))
        self.in_XZ.setText(QCoreApplication.translate("DMatLib", u"XZ", None))
        self.in_Y.setText(QCoreApplication.translate("DMatLib", u"Y", None))
        self.in_X.setText(QCoreApplication.translate("DMatLib", u"X", None))
        self.in_YZ.setText(QCoreApplication.translate("DMatLib", u"YZ", None))
        self.in_Z.setText(QCoreApplication.translate("DMatLib", u"Z", None))
        self.g_mag.setTitle(QCoreApplication.translate("DMatLib", u"Magnetics", None))
        self.out_mur_lin.setText(
            QCoreApplication.translate("DMatLib", u"mur_lin = 0.555", None)
        )
        self.out_Brm20.setText(
            QCoreApplication.translate("DMatLib", u"Brm20 = 1 T", None)
        )
        self.out_alpha_Br.setText(
            QCoreApplication.translate("DMatLib", u"alpha_Br=0.1", None)
        )
        self.out_wlam.setText(
            QCoreApplication.translate("DMatLib", u"wlam = 0.5 m", None)
        )
        self.in_BH_curve.setText(
            QCoreApplication.translate("DMatLib", u"BH curve:", None)
        )
        self.out_BH.setText(QCoreApplication.translate("DMatLib", u"-", None))

    # retranslateUi



def _search_(obj, convert_list):
    # V 1.0.4 => 1.1.0: New definition for LamSlotMag + SlotMag
    if (
        "__class__" in obj.keys()
        and obj["__class__"] == "MachineSIPMSM"
        and "magnet" not in obj["rotor"]
    ):
        # add to list for later conversion
        convert_list.append(obj)
    else:
        # walk through the dict
        for value in obj.values():
            if isinstance(value, dict):
                # recursively search the dict
                _search_(value, convert_list)
            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict):
                        # recursively search the dict
                        _search_(item, convert_list)


def convert_init_dict(init_dict):
    """Convert an init_dict from an old version of pyleecan to the current one"""
    convert_list = []
    _search_(init_dict, convert_list)

    # V 1.0.4 => 1.1.0: New definition for LamSlotMag + SlotMag
    for machine_dict in convert_list:
        print("Old machine version detected, Updating the LamSlotMag object")
        # readability
        rotor = machine_dict["rotor"]
        slot = machine_dict["rotor"]["slot"]

        # Moving the magnet (use only one magnet)
        if len(slot["magnet"]) > 1:
            print(
                "LamSlotMag with more than one magnet per pole "
                + "is not available for now. Only keeping first magnet."
            )
        rotor["magnet"] = slot["magnet"][0]
        slot.pop("magnet")

        # Update the slot with the magnet parameters
        if rotor["magnet"] is not None:
            slot["__class__"] = "SlotM" + rotor["magnet"]["__class__"][-2:]
            slot["Wmag"] = rotor["magnet"]["Wmag"]
            slot["Hmag"] = rotor["magnet"]["Hmag"]
            if "Rtop" in rotor["magnet"]:
                slot["Rtopm"] = rotor["magnet"]["Rtop"]
            rotor["magnet"]["__class__"] = "Magnet"



# -*- coding: utf-8 -*-

from numpy import pi, arcsin


def comp_angle_opening(self):
    """Compute the average opening angle of the Slot

    Parameters
    ----------
    self : SlotW61
        A SlotW61 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    slot_pitch = 2 * pi / self.Zs
    tooth_angle = float(2 * arcsin(self.W0 / (2 * self.get_Rbo())))

    return slot_pitch - tooth_angle



# -*- coding: utf-8 -*-
def comp_length(self):
    """Compute the length of the SurfLine object

    Parameters
    ----------
    self : SurfLine
        A SurfLine object

    Returns
    -------
    length: float
        Length of the surface [m]


    """
    # check if the SurfLine is correct
    self.check()
    length = 0
    for line in self.line_list:
        length += line.comp_length()

    return length



import matplotlib.pyplot as plt
from numpy import pi, exp

from ....Classes.Arc1 import Arc1
from ....Classes.LamHole import LamHole
from ....Classes.Segment import Segment
from ....definitions import config_dict
from ....Functions.Plot import (
    ARROW_COLOR,
    ARROW_WIDTH,
    MAIN_LINE_COLOR,
    MAIN_LINE_STYLE,
    MAIN_LINE_WIDTH,
    P_FONT_SIZE,
    SC_FONT_SIZE,
    SC_LINE_COLOR,
    SC_LINE_STYLE,
    SC_LINE_WIDTH,
    TEXT_BOX,
    plot_quote,
)
from ....Methods import ParentMissingError

MAGNET_COLOR = config_dict["PLOT"]["COLOR_DICT"]["MAGNET_COLOR"]


def plot_schematics(
    self,
    is_default=False,
    is_add_point_label=False,
    is_add_schematics=True,
    is_add_main_line=True,
    type_add_active=True,
    save_path=None,
    is_show_fig=True,
):
    """Plot the schematics of the slot

    Parameters
    ----------
    self : HoleM50
        A HoleM50 object
    is_default : bool
        True: plot default schematics, else use current slot values
    is_add_point_label : bool
        True to display the name of the points (Z1, Z2....)
    is_add_schematics : bool
        True to display the schematics information (W0, H0...)
    is_add_main_line : bool
        True to display "main lines" (slot opening and 0x axis)
    type_add_active : int
        0: No active surface, 1: active surface as winding, 2: active surface as magnet
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    is_show_fig : bool
        To call show at the end of the method
    """

    # Use some default parameter
    if is_default:
        hole = type(self)(
            H0=0.01496,
            H1=0.0065,
            H2=0.003,
            H3=0.0085,
            H4=0.004,
            W0=0.042,
            W1=0.005,
            W2=0.004,
            W3=0.007,
            W4=0.0129,
            Zh=8,
        )
        lam = LamHole(
            Rint=0.05532, Rext=0.0812, is_internal=True, is_stator=False, hole=[hole]
        )
        hole.plot_schematics(
            is_default=False,
            is_add_point_label=is_add_point_label,
            is_add_schematics=is_add_schematics,
            is_add_main_line=is_add_main_line,
            type_add_active=type_add_active,
            save_path=save_path,
            is_show_fig=is_show_fig,
        )
    elif type_add_active == 0:
        # Remove magnets
        lam = self.parent.copy()
        lam.hole[0].remove_magnet()
        lam.hole[0].plot_schematics(
            is_default=False,
            is_add_point_label=is_add_point_label,
            is_add_schematics=is_add_schematics,
            is_add_main_line=is_add_main_line,
            type_add_active=2,
            save_path=save_path,
            is_show_fig=is_show_fig,
        )
    else:
        # Getting the main plot
        if self.parent is None:
            raise ParentMissingError("Error: The hole is not inside a Lamination")
        lam = self.parent
        alpha = pi / 2  # To rotate the schematics
        lam.plot(
            alpha=pi / self.Zh + alpha,
            is_show_fig=False,
            is_lam_only=type_add_active == 0,
        )  # center hole on Ox axis
        sp = 2 * pi / self.Zh
        Rbo = self.get_Rbo()
        fig = plt.gcf()
        ax = plt.gca()
        point_dict = self._comp_point_coordinate()

        # Adding point label
        if is_add_point_label:
            for name, Z in point_dict.items():
                Z = Z * exp(1j * alpha)
                ax.text(
                    Z.real,
                    Z.imag,
                    name,
                    fontsize=P_FONT_SIZE,
                    bbox=TEXT_BOX,
                )

        # Adding schematics
        if is_add_schematics:
            # W0
            line = Segment(
                point_dict["Z9"] * exp(1j * alpha), point_dict["Z9s"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="W0",
                offset_label=self.W0 * 0.3 + 1j * self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # W1
            line = Segment(
                (point_dict["Z8"] + point_dict["Z7"]) / 2 * exp(1j * alpha),
                (point_dict["Z8s"] + point_dict["Z7s"]) / 2 * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="W1",
                offset_label=-1j * self.H3 * 0.4,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # W2
            line = Segment(
                point_dict["Z8"] * exp(1j * alpha), point_dict["Z8b"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="W2",
                offset_label=1j * self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # W3
            line = Segment(
                (point_dict["Z1s"] + point_dict["Z11s"]) * 0.5 * exp(1j * alpha),
                (point_dict["Z1"] + point_dict["Z11"])
                * 0.5
                * exp(1j * sp)
                * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="W3",
                offset_label=-1j * self.H3 * 0.3,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # W4
            line = Segment(
                point_dict["Z5"] * exp(1j * alpha), point_dict["Z4"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="W4",
                offset_label=-1j * self.H3 * 0.3,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # H0
            line = Segment(
                Rbo * exp(1j * alpha), point_dict["Z8"].real * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="H0",
                offset_label=self.W0 * 0.05 + 1j * self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # H1
            line = Segment(
                Rbo * exp(-1j * sp / 2) * exp(1j * alpha),
                (Rbo - self.H1) * exp(-1j * sp / 2) * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="H1",
                offset_label=self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # H2
            line = Segment(
                (point_dict["Z3s"] + point_dict["Z6s"]) * 0.5 * exp(1j * alpha),
                (point_dict["Z4s"] + point_dict["Z5s"]) * 0.5 * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="H2",
                offset_label=self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # H3
            line = Segment(
                point_dict["Z4s"] * exp(1j * alpha),
                point_dict["Z8cs"] * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="H3",
                offset_label=self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # H4
            line = Segment(
                point_dict["Z9s"] * exp(1j * alpha),
                point_dict["Z10s"] * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="H4",
                offset_label=self.H3 * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )

        if is_add_main_line:
            # Ox axis
            line = Segment(0, lam.Rext * 1.5 * exp(1j * alpha))
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # Tooth axis
            line = Segment(0, lam.Rext * 1.5 * exp(1j * sp / 2) * exp(1j * alpha))
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            line = Segment(0, lam.Rext * 1.5 * exp(-1j * sp / 2) * exp(1j * alpha))
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # H1 radius
            line = Arc1(
                begin=(Rbo - self.H1) * exp(-1j * pi / 2 * 0.9) * exp(1j * alpha),
                end=(Rbo - self.H1) * exp(1j * pi / 2 * 0.9) * exp(1j * alpha),
                radius=Rbo - self.H1,
                is_trigo_direction=True,
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # H4 radius
            line = Arc1(
                begin=(Rbo - self.H1 + self.H4)
                * exp(-1j * pi / 2 * 0.9)
                * exp(1j * alpha),
                end=(Rbo - self.H1 + self.H4)
                * exp(1j * pi / 2 * 0.9)
                * exp(1j * alpha),
                radius=Rbo - self.H1 + self.H4,
                is_trigo_direction=True,
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # W1 lines
            line = Segment(
                point_dict["Z7"] * exp(1j * alpha), point_dict["Z7s"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            line = Segment(
                point_dict["Z8"] * exp(1j * alpha), point_dict["Z8s"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # W2 lines
            line = Segment(
                point_dict["Z8"] * exp(1j * alpha), point_dict["Z8a"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            line = Segment(
                point_dict["Z8s"] * exp(1j * alpha),
                point_dict["Z8as"] * exp(1j * alpha),
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # H2 lines
            line = Segment(
                point_dict["Z3s"] * exp(1j * alpha), point_dict["Z6s"] * exp(1j * alpha)
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )

        # Zooming and cleaning
        W = abs(point_dict["Z11s"].imag) * 1.3
        Rint = self.parent.Rint
        Rext = self.parent.Rext

        plt.axis("equal")
        ax.set_ylim(Rint, Rext)
        ax.set_xlim(-W, W)
        fig.canvas.set_window_title(type(self).__name__ + " Schematics")
        ax.set_title("")
        ax.get_legend().remove()
        ax.set_axis_off()

        # Save / Show
        if save_path is not None:
            fig.savefig(save_path)
            plt.close()

        if is_show_fig:
            fig.show()



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch
from matplotlib.pyplot import axis

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

FRAME_COLOR = config_dict["PLOT"]["COLOR_DICT"]["FRAME_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_show_fig=True,
):
    """Plot the Frame in a matplotlib fig

    Parameters
    ----------
    self : Frame
        A Frame object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    sym : int
        Symmetry factor (1= plot full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None

    """

    # plot only if the frame has a height >0
    if self.comp_height_eq() != 0:
        (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")
        surf_list = self.build_geometry(sym=sym, alpha=alpha, delta=delta)
        patches = list()
        for surf in surf_list:
            patches.extend(
                surf.get_patches(color=FRAME_COLOR, is_edge_only=is_edge_only)
            )

        axes.set_xlabel("(m)")
        axes.set_ylabel("(m)")
        axes.set_title("Frame")
        for patch in patches:
            axes.add_patch(patch)
        axis("equal")

        # The Lamination is centered in the figure
        Lim = self.Rext * 1.1
        axes.set_xlim(-Lim, Lim, auto=True)
        axes.set_ylim(-Lim, Lim, auto=True)

        if not is_edge_only:
            patch_leg.append(Patch(color=FRAME_COLOR))
            label_leg.append("Frame")
            axes.legend(patch_leg, label_leg)
        if is_show_fig:
            fig.show()



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface(self):
    """Compute the surface of the conductor

    Parameters
    ----------
    self : CondType12
        A CondType12 object

    Returns
    -------
    S: float
        Surface of the conductor (with insulation) [m**2]

    """

    return pi * ((self.Wins_cond / 2.0) ** 2)



# -*- coding: utf-8 -*-

from numpy import pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW23 import *


def check(self):
    """Check that the SlotW23 object is correct

    Parameters
    ----------
    self : SlotW23
        A SlotW23 object

    Returns
    -------
    None

    Raises
    -------
    S23_W01CheckError
        You must have W0 <= W1
    S23_H1rCheckError
        With H1 in radian, you must have H1 < pi/2

    """
    if self.W1 is not None and self.W1 < self.W0:
        raise S23_W01CheckError("You must have W0 <= W1")

    if self.H1_is_rad and self.H1 >= pi / 2:
        raise S23_H1rCheckError("With H1 in radian, you must have H1 < pi/2")



# -*- coding: utf-8 -*-

from ....Methods.Machine.LamSlotMulti import LamSlotMultiAlphaError
from numpy import pi


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self : LamSlotMulti
        A LamSlotMulti object

    Returns
    -------
    None
    """

    super(type(self), self).check()

    # Check all the slots
    for slot in self.slot_list:
        slot.check()

    #
    if len(self.alpha.shape) > 1:
        raise LamSlotMultiAlphaError("Alpha should have be a vector (1D array)")
    if len(self.alpha) != len(self.slot_list):
        raise LamSlotMultiAlphaError("Alpha should have the same length as slot_list")
    if self.alpha[0] < 0:
        raise LamSlotMultiAlphaError(
            "Alpha should be an ordered array with values between 0 and 2*pi"
        )
    for ii in range(len(self.alpha) - 1):
        if self.alpha[ii] > self.alpha[ii + 1]:
            raise LamSlotMultiAlphaError(
                "Alpha should be an ordered array with values between 0 and 2*pi"
            )
    if self.alpha[-1] > 2 * pi:
        raise LamSlotMultiAlphaError(
            "Alpha should be an ordered array with values between 0 and 2*pi"
        )



# -*- coding: utf-8 -*-

from numpy import exp, pi, angle, abs as np_abs


def is_on_line(self, Z):
    """Check is a point defined by its complex coordinate is on the arc

    Parameters
    ----------
    self : Arc
        An Arc object
    Z : complex
        Complex coordinate of the point

    Returns
    -------
    is_on_line : bool
        True if the point is on the arc
    """

    Zc = self.get_center()
    R = self.comp_radius()

    # Check if on the circle
    if abs(np_abs(Z - Zc) - R) > 1e-6:
        return False

    # Check if point is begin or end
    begin = self.get_begin()
    end = self.get_end()
    if np_abs(Z - begin) < 1e-6:
        return True
    if np_abs(Z - end) < 1e-6:
        return True

    # Go to the coordinate system Zc as center, begin on X > 0 axis
    Ze = (end - Zc) * exp(-1j * angle(begin - Zc))
    Z = (Z - Zc) * exp(-1j * angle(begin - Zc))

    alpha = self.get_angle()
    Ae = angle(Ze) % (2 * pi)
    Az = angle(Z) % (2 * pi)
    if alpha > 0:  # Trigo direction
        if Ae > Az:
            return True
        return False
    # Clockwise direction
    if Az > Ae:
        return True
    return False



# -*- coding: utf-8 -*-
import collections
import numpy as np


def get_cell(self, indices=None):
    """Return the connectivity for one selected element

    Parameters
    ----------
    self : MeshMat
        an MeshMat object
    indices : list
        list of indice. If None, return all.

    Returns
    -------
    cells: dict
        Dict of connectivities

    """
    if not isinstance(indices, collections.Iterable):
        indices = (indices,)

    cells = dict()
    indice_dict = dict()
    nb_cell = 0
    for key in self.cell:
        cells[key] = list()
        indice_dict[key] = list()

        for ind in indices:
            connect = self.cell[key].get_connectivity(ind)
            if connect is not None:
                cells[key].append(connect)
                nb_cell = nb_cell + len(connect)
                indice_dict[key].append(ind)

        cells[key] = np.squeeze(np.array(cells[key]))

    return cells, nb_cell, indice_dict



# -*- coding: utf-8 -*-

import numpy as np


def jacobian(self, point, vertice):
    """Compute jacobian, jacobian determinant and jacobian derivatives for linear triangle.

    Parameters
    ----------
    :param self : a RefElement object

    Returns
    -------

    """

    grad_func = self.grad_shape_function(point)
    jacob = np.dot(grad_func, vertice[:, 0:2])
    det_jacob = np.linalg.det(jacob)

    return jacob, det_jacob



# -*- coding: utf-8 -*-

from numpy import linspace, sin, pi


def get_data(self):
    """Generate the sinus vector

    Parameters
    ----------
    self : ImportGenVectSin
        An ImportGenVectSin object

    Returns
    -------
    vect: ndarray
        The generated sinus vector

    """

    time = linspace(start=0, stop=self.Tf, num=self.N, endpoint=False)

    vect = self.A * sin(2 * pi * self.f * time + self.Phi)
    return self.edit_matrix(vect)



# -*- coding: utf-8 -*-
import numpy as np

from pyleecan.Functions.make_ndarray_equal import make_ndarray_equal


def get_axes_list(self):
    """Get the axis of variables stored in Solution.

    Parameters
    ----------
    self : SolutionVector
        an SolutionVector object
    field_name : str
        name of the field to return

    Returns
    -------
    field: array
        an array of field values

    """

    # Build axis list
    ax_name = list()
    ax_size = list()

    comp = self.field.components["comp_x"]
    for axis in comp.axes:
        ax_name.append(axis.name)
        ax_size.append(axis.get_length())

    ax_name.append("component")
    ax_size.append(len(self.field.components))

    return ax_name, ax_size



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.LamSlot import LamSlot


def comp_surfaces(self):
    """Compute the Lamination surfaces (Slam, Svent, Sslot, Smag)

    Parameters
    ----------
    self : LamSlotMag
        A LamSlotMag object

    Returns
    -------
    S_dict: dict
        Lamination surface dictionnary (Slam, Svent, Sslot, Smag) [m**2]

    """

    S_dict = LamSlot.comp_surfaces(self)
    Smag = self.slot.comp_surface_active()

    S_dict["Smag"] = Smag

    return S_dict



def set_FEMM_wind_material(femm, materials, cname, Jcus=0, Cduct=None, dwire=None):
    """Create or update the property of a winding material

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    materials: list
        list the name of all materials
    cname: str
        name of the material to create/update
    Jcus : float
        Applied source current density [A/mm^2]
    Cduct : float
        Electrical conductivity of the material [MS/m]
    dwire : float
        Diameter of each of the wire’s constituent strand [mm]

    Returns
    -------
    circuits : list
        list the name of the circuits in FEMM

    """
    if cname not in materials:
        # Create a new material
        femm.mi_addmaterial(
            cname,
            1,  # Relative permeability in the x- or r-direction
            1,  # Relative permeability in the y- or z-direction
            0,  # Permanent magnet coercivity [A/m]
            Jcus,
            Cduct,
            0,  # Lamination thickness [mm]
            0,  # Hysteresis lag angle [deg], (used for nonlinear BH curves)
            1,  # Fraction of the volume occupied per lamination that is actually filled withiron
            0,  # LamType (Not laminated)
            0,  # Hysteresis lag [deg] in the x-direction for linear problems
            0,  # Hysteresis lag [deg] in the y-direction for linear problems
            1,  # Number of strands in the wire build
            dwire,
        )
        materials.append(cname)

    return materials



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.LamSlot import LamSlot


def comp_surfaces(self):
    """Compute the Lamination surfaces (Slam, Svent, Sslot, Swind)

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    S_dict: dict
        Lamination surface dictionnary (Slam, Svent, Sslot, Swind) [m**2]

    """

    S_dict = LamSlot.comp_surfaces(self)

    if self.slot is not None:
        S_dict["Swind"] = self.get_Zs() * self.slot.comp_surface_active()
    else:
        S_dict["Swind"] = 0

    return S_dict



# -*- coding: utf-8 -*-


def get_mesh_param(label, FEMM_dict):
    """Returns main mesh parameters corresponding to a surface label
    Parameters
    ----------
    label : str
        label of the surface to assign
    FEMM_dict : dict
        Dictionnary containing the main parameters of FEMM
    Returns
    -------
    mesh_dict : dict
        dictionnary containing the main mesh parameters of the surface
    """

    mesh_dict = dict()

    # Default automesh except airgap
    mesh_dict["automesh"] = FEMM_dict["automesh"]

    if "Lamination_Stator" in label:  # Stator
        if "Bore" in label:
            mesh_dict["element_size"] = FEMM_dict["elementsize_slotS"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_slotS"]
        else:
            mesh_dict["element_size"] = FEMM_dict["elementsize_yokeS"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_yokeS"]
        mesh_dict["group"] = FEMM_dict["groups"]["GROUP_SC"]
    elif "Lamination_Rotor" in label:  # Rotor
        if "Bore" in label:
            mesh_dict["element_size"] = FEMM_dict["elementsize_slotR"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_slotR"]
        else:
            mesh_dict["element_size"] = FEMM_dict["elementsize_yokeR"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_yokeR"]
        mesh_dict["group"] = FEMM_dict["groups"]["GROUP_RC"]
    elif "Ventilation" in label:  # Ventilation
        mesh_dict["element_size"] = FEMM_dict["maxelementsize"]
        mesh_dict["meshsize"] = FEMM_dict["meshsize_air"]
        if "Stator" in label:  # if the Ventilation is on the Stator
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_SV"]
        else:  # if the Ventilation is on the Rotor
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_RV"]
    elif "Hole_" in label:
        mesh_dict["element_size"] = FEMM_dict["maxelementsize"]
        mesh_dict["meshsize"] = FEMM_dict["meshsize_air"]
        if "Stator" in label:  # if the Hole is on the Stator
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_SH"]
        else:  # if the Hole is on the Rotor
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_RH"]
    elif "Wind" in label or "Bar" in label:  # Winding on the Lamination
        if "Stator" in label:  # if the winding is on the Stator
            mesh_dict["element_size"] = FEMM_dict["elementsize_slotS"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_slotS"]
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_SW"]
        else:  # if the winding is on the Rotor
            mesh_dict["element_size"] = FEMM_dict["elementsize_slotR"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_slotR"]
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_RW"]
    elif "Magnet" in label:  # Magnet
        if "Stator" in label:  # if the Magnet is on the Stator
            mesh_dict["element_size"] = FEMM_dict["elementsize_magnetS"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_magnetS"]
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_SM"]
        else:  # if the Magnet is on the Rotor
            mesh_dict["element_size"] = FEMM_dict["elementsize_magnetR"]
            mesh_dict["meshsize"] = FEMM_dict["meshsize_magnetR"]
            mesh_dict["group"] = FEMM_dict["groups"]["GROUP_RM"]
    elif "airgap" in label.lower() or "sliding" in label:
        mesh_dict["automesh"] = FEMM_dict["automesh_airgap"]
        mesh_dict["element_size"] = FEMM_dict["elementsize_airgap"]
        mesh_dict["meshsize"] = FEMM_dict["meshsize_airgap"]
        mesh_dict["group"] = FEMM_dict["groups"]["GROUP_AG"]
    elif "No_mesh" in label:
        mesh_dict["automesh"] = 0
        mesh_dict["element_size"] = 0
        mesh_dict["meshsize"] = 0
        mesh_dict["group"] = FEMM_dict["groups"]["GROUP_AG"]
    elif "Stator" in label:  # if label don't belong to any of the other groups
        mesh_dict["element_size"] = FEMM_dict["maxelementsize"]
        mesh_dict["group"] = FEMM_dict["groups"]["GROUP_SC"]
    elif "Rotor" in label:  # if label don't belong to any of the other groups
        mesh_dict["element_size"] = FEMM_dict["maxelementsize"]
        mesh_dict["group"] = FEMM_dict["groups"]["GROUP_RC"]
    return mesh_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/Output.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/Output
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Output.Output.getter.get_angle_offset_initial import (
        get_angle_offset_initial,
    )
except ImportError as error:
    get_angle_offset_initial = error

try:
    from ..Methods.Output.Output.getter.get_angle_rotor import get_angle_rotor
except ImportError as error:
    get_angle_rotor = error

try:
    from ..Methods.Output.Output.getter.get_BH_rotor import get_BH_rotor
except ImportError as error:
    get_BH_rotor = error

try:
    from ..Methods.Output.Output.getter.get_BH_stator import get_BH_stator
except ImportError as error:
    get_BH_stator = error

try:
    from ..Methods.Output.Output.getter.get_path_result import get_path_result
except ImportError as error:
    get_path_result = error

try:
    from ..Methods.Output.Output.getter.get_machine_periodicity import (
        get_machine_periodicity,
    )
except ImportError as error:
    get_machine_periodicity = error

try:
    from ..Methods.Output.Output.getter.get_rot_dir import get_rot_dir
except ImportError as error:
    get_rot_dir = error

try:
    from ..Methods.Output.Output.plot.plot_2D_Data import plot_2D_Data
except ImportError as error:
    plot_2D_Data = error

try:
    from ..Methods.Output.Output.getter.get_fund_harm import get_fund_harm
except ImportError as error:
    get_fund_harm = error

try:
    from ..Methods.Output.Output.plot.plot_A_quiver_2D import plot_A_quiver_2D
except ImportError as error:
    plot_A_quiver_2D = error

try:
    from ..Methods.Output.Output.plot.plot_3D_Data import plot_3D_Data
except ImportError as error:
    plot_3D_Data = error

try:
    from ..Methods.Output.Output.plot.plot_A_time_space import plot_A_time_space
except ImportError as error:
    plot_A_time_space = error

try:
    from ..Methods.Output.Output.getter.get_data_from_str import get_data_from_str
except ImportError as error:
    get_data_from_str = error

try:
    from ..Methods.Output.Output.print_memory import print_memory
except ImportError as error:
    print_memory = error


from ._check import InitUnKnowClassError
from .Simulation import Simulation
from .OutGeo import OutGeo
from .OutElec import OutElec
from .OutMag import OutMag
from .OutStruct import OutStruct
from .OutPost import OutPost
from .OutForce import OutForce
from .OutLoss import OutLoss


class Output(FrozenClass):
    """Main Output object: gather all the outputs of all the modules"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Output.Output.getter.get_angle_offset_initial
    if isinstance(get_angle_offset_initial, ImportError):
        get_angle_offset_initial = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_angle_offset_initial: "
                    + str(get_angle_offset_initial)
                )
            )
        )
    else:
        get_angle_offset_initial = get_angle_offset_initial
    # cf Methods.Output.Output.getter.get_angle_rotor
    if isinstance(get_angle_rotor, ImportError):
        get_angle_rotor = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_angle_rotor: " + str(get_angle_rotor)
                )
            )
        )
    else:
        get_angle_rotor = get_angle_rotor
    # cf Methods.Output.Output.getter.get_BH_rotor
    if isinstance(get_BH_rotor, ImportError):
        get_BH_rotor = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_BH_rotor: " + str(get_BH_rotor)
                )
            )
        )
    else:
        get_BH_rotor = get_BH_rotor
    # cf Methods.Output.Output.getter.get_BH_stator
    if isinstance(get_BH_stator, ImportError):
        get_BH_stator = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_BH_stator: " + str(get_BH_stator)
                )
            )
        )
    else:
        get_BH_stator = get_BH_stator
    # cf Methods.Output.Output.getter.get_path_result
    if isinstance(get_path_result, ImportError):
        get_path_result = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_path_result: " + str(get_path_result)
                )
            )
        )
    else:
        get_path_result = get_path_result
    # cf Methods.Output.Output.getter.get_machine_periodicity
    if isinstance(get_machine_periodicity, ImportError):
        get_machine_periodicity = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_machine_periodicity: "
                    + str(get_machine_periodicity)
                )
            )
        )
    else:
        get_machine_periodicity = get_machine_periodicity
    # cf Methods.Output.Output.getter.get_rot_dir
    if isinstance(get_rot_dir, ImportError):
        get_rot_dir = property(
            fget=lambda x: raise_(
                ImportError("Can't use Output method get_rot_dir: " + str(get_rot_dir))
            )
        )
    else:
        get_rot_dir = get_rot_dir
    # cf Methods.Output.Output.plot.plot_2D_Data
    if isinstance(plot_2D_Data, ImportError):
        plot_2D_Data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method plot_2D_Data: " + str(plot_2D_Data)
                )
            )
        )
    else:
        plot_2D_Data = plot_2D_Data
    # cf Methods.Output.Output.getter.get_fund_harm
    if isinstance(get_fund_harm, ImportError):
        get_fund_harm = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_fund_harm: " + str(get_fund_harm)
                )
            )
        )
    else:
        get_fund_harm = get_fund_harm
    # cf Methods.Output.Output.plot.plot_A_quiver_2D
    if isinstance(plot_A_quiver_2D, ImportError):
        plot_A_quiver_2D = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method plot_A_quiver_2D: " + str(plot_A_quiver_2D)
                )
            )
        )
    else:
        plot_A_quiver_2D = plot_A_quiver_2D
    # cf Methods.Output.Output.plot.plot_3D_Data
    if isinstance(plot_3D_Data, ImportError):
        plot_3D_Data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method plot_3D_Data: " + str(plot_3D_Data)
                )
            )
        )
    else:
        plot_3D_Data = plot_3D_Data
    # cf Methods.Output.Output.plot.plot_A_time_space
    if isinstance(plot_A_time_space, ImportError):
        plot_A_time_space = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method plot_A_time_space: "
                    + str(plot_A_time_space)
                )
            )
        )
    else:
        plot_A_time_space = plot_A_time_space
    # cf Methods.Output.Output.getter.get_data_from_str
    if isinstance(get_data_from_str, ImportError):
        get_data_from_str = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method get_data_from_str: "
                    + str(get_data_from_str)
                )
            )
        )
    else:
        get_data_from_str = get_data_from_str
    # cf Methods.Output.Output.print_memory
    if isinstance(print_memory, ImportError):
        print_memory = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Output method print_memory: " + str(print_memory)
                )
            )
        )
    else:
        print_memory = print_memory
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        simu=-1,
        path_result="",
        geo=-1,
        elec=-1,
        mag=-1,
        struct=-1,
        post=-1,
        logger_name="Pyleecan.Output",
        force=-1,
        loss=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "simu" in list(init_dict.keys()):
                simu = init_dict["simu"]
            if "path_result" in list(init_dict.keys()):
                path_result = init_dict["path_result"]
            if "geo" in list(init_dict.keys()):
                geo = init_dict["geo"]
            if "elec" in list(init_dict.keys()):
                elec = init_dict["elec"]
            if "mag" in list(init_dict.keys()):
                mag = init_dict["mag"]
            if "struct" in list(init_dict.keys()):
                struct = init_dict["struct"]
            if "post" in list(init_dict.keys()):
                post = init_dict["post"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "force" in list(init_dict.keys()):
                force = init_dict["force"]
            if "loss" in list(init_dict.keys()):
                loss = init_dict["loss"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.simu = simu
        self.path_result = path_result
        self.geo = geo
        self.elec = elec
        self.mag = mag
        self.struct = struct
        self.post = post
        self.logger_name = logger_name
        self.force = force
        self.loss = loss

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Output_str = ""
        if self.parent is None:
            Output_str += "parent = None " + linesep
        else:
            Output_str += "parent = " + str(type(self.parent)) + " object" + linesep
        if self.simu is not None:
            tmp = self.simu.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "simu = " + tmp
        else:
            Output_str += "simu = None" + linesep + linesep
        Output_str += 'path_result = "' + str(self.path_result) + '"' + linesep
        if self.geo is not None:
            tmp = self.geo.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "geo = " + tmp
        else:
            Output_str += "geo = None" + linesep + linesep
        if self.elec is not None:
            tmp = self.elec.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "elec = " + tmp
        else:
            Output_str += "elec = None" + linesep + linesep
        if self.mag is not None:
            tmp = self.mag.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "mag = " + tmp
        else:
            Output_str += "mag = None" + linesep + linesep
        if self.struct is not None:
            tmp = self.struct.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "struct = " + tmp
        else:
            Output_str += "struct = None" + linesep + linesep
        if self.post is not None:
            tmp = self.post.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "post = " + tmp
        else:
            Output_str += "post = None" + linesep + linesep
        Output_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        if self.force is not None:
            tmp = self.force.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "force = " + tmp
        else:
            Output_str += "force = None" + linesep + linesep
        if self.loss is not None:
            tmp = self.loss.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Output_str += "loss = " + tmp
        else:
            Output_str += "loss = None" + linesep + linesep
        return Output_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.simu != self.simu:
            return False
        if other.path_result != self.path_result:
            return False
        if other.geo != self.geo:
            return False
        if other.elec != self.elec:
            return False
        if other.mag != self.mag:
            return False
        if other.struct != self.struct:
            return False
        if other.post != self.post:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.force != self.force:
            return False
        if other.loss != self.loss:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.simu is None and self.simu is not None) or (
            other.simu is not None and self.simu is None
        ):
            diff_list.append(name + ".simu None mismatch")
        elif self.simu is not None:
            diff_list.extend(self.simu.compare(other.simu, name=name + ".simu"))
        if other._path_result != self._path_result:
            diff_list.append(name + ".path_result")
        if (other.geo is None and self.geo is not None) or (
            other.geo is not None and self.geo is None
        ):
            diff_list.append(name + ".geo None mismatch")
        elif self.geo is not None:
            diff_list.extend(self.geo.compare(other.geo, name=name + ".geo"))
        if (other.elec is None and self.elec is not None) or (
            other.elec is not None and self.elec is None
        ):
            diff_list.append(name + ".elec None mismatch")
        elif self.elec is not None:
            diff_list.extend(self.elec.compare(other.elec, name=name + ".elec"))
        if (other.mag is None and self.mag is not None) or (
            other.mag is not None and self.mag is None
        ):
            diff_list.append(name + ".mag None mismatch")
        elif self.mag is not None:
            diff_list.extend(self.mag.compare(other.mag, name=name + ".mag"))
        if (other.struct is None and self.struct is not None) or (
            other.struct is not None and self.struct is None
        ):
            diff_list.append(name + ".struct None mismatch")
        elif self.struct is not None:
            diff_list.extend(self.struct.compare(other.struct, name=name + ".struct"))
        if (other.post is None and self.post is not None) or (
            other.post is not None and self.post is None
        ):
            diff_list.append(name + ".post None mismatch")
        elif self.post is not None:
            diff_list.extend(self.post.compare(other.post, name=name + ".post"))
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        if (other.force is None and self.force is not None) or (
            other.force is not None and self.force is None
        ):
            diff_list.append(name + ".force None mismatch")
        elif self.force is not None:
            diff_list.extend(self.force.compare(other.force, name=name + ".force"))
        if (other.loss is None and self.loss is not None) or (
            other.loss is not None and self.loss is None
        ):
            diff_list.append(name + ".loss None mismatch")
        elif self.loss is not None:
            diff_list.extend(self.loss.compare(other.loss, name=name + ".loss"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.simu)
        S += getsizeof(self.path_result)
        S += getsizeof(self.geo)
        S += getsizeof(self.elec)
        S += getsizeof(self.mag)
        S += getsizeof(self.struct)
        S += getsizeof(self.post)
        S += getsizeof(self.logger_name)
        S += getsizeof(self.force)
        S += getsizeof(self.loss)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Output_dict = dict()
        if self.simu is None:
            Output_dict["simu"] = None
        else:
            Output_dict["simu"] = self.simu.as_dict(**kwargs)
        Output_dict["path_result"] = self.path_result
        if self.geo is None:
            Output_dict["geo"] = None
        else:
            Output_dict["geo"] = self.geo.as_dict(**kwargs)
        if self.elec is None:
            Output_dict["elec"] = None
        else:
            Output_dict["elec"] = self.elec.as_dict(**kwargs)
        if self.mag is None:
            Output_dict["mag"] = None
        else:
            Output_dict["mag"] = self.mag.as_dict(**kwargs)
        if self.struct is None:
            Output_dict["struct"] = None
        else:
            Output_dict["struct"] = self.struct.as_dict(**kwargs)
        if self.post is None:
            Output_dict["post"] = None
        else:
            Output_dict["post"] = self.post.as_dict(**kwargs)
        Output_dict["logger_name"] = self.logger_name
        if self.force is None:
            Output_dict["force"] = None
        else:
            Output_dict["force"] = self.force.as_dict(**kwargs)
        if self.loss is None:
            Output_dict["loss"] = None
        else:
            Output_dict["loss"] = self.loss.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        Output_dict["__class__"] = "Output"
        return Output_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.simu is not None:
            self.simu._set_None()
        self.path_result = None
        if self.geo is not None:
            self.geo._set_None()
        if self.elec is not None:
            self.elec._set_None()
        if self.mag is not None:
            self.mag._set_None()
        if self.struct is not None:
            self.struct._set_None()
        if self.post is not None:
            self.post._set_None()
        self.logger_name = None
        if self.force is not None:
            self.force._set_None()
        if self.loss is not None:
            self.loss._set_None()

    def _get_simu(self):
        """getter of simu"""
        return self._simu

    def _set_simu(self, value):
        """setter of simu"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "simu")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Simulation()
        check_var("simu", value, "Simulation")
        self._simu = value

        if self._simu is not None:
            self._simu.parent = self

    simu = property(
        fget=_get_simu,
        fset=_set_simu,
        doc=u"""Simulation object that generated the Output

        :Type: Simulation
        """,
    )

    def _get_path_result(self):
        """getter of path_result"""
        return self._path_result

    def _set_path_result(self, value):
        """setter of path_result"""
        check_var("path_result", value, "str")
        self._path_result = value

    path_result = property(
        fget=_get_path_result,
        fset=_set_path_result,
        doc=u"""Path to the folder to same the results

        :Type: str
        """,
    )

    def _get_geo(self):
        """getter of geo"""
        return self._geo

    def _set_geo(self, value):
        """setter of geo"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "geo")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutGeo()
        check_var("geo", value, "OutGeo")
        self._geo = value

        if self._geo is not None:
            self._geo.parent = self

    geo = property(
        fget=_get_geo,
        fset=_set_geo,
        doc=u"""Geometry output

        :Type: OutGeo
        """,
    )

    def _get_elec(self):
        """getter of elec"""
        return self._elec

    def _set_elec(self, value):
        """setter of elec"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "elec")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutElec()
        check_var("elec", value, "OutElec")
        self._elec = value

        if self._elec is not None:
            self._elec.parent = self

    elec = property(
        fget=_get_elec,
        fset=_set_elec,
        doc=u"""Electrical module output

        :Type: OutElec
        """,
    )

    def _get_mag(self):
        """getter of mag"""
        return self._mag

    def _set_mag(self, value):
        """setter of mag"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "mag")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutMag()
        check_var("mag", value, "OutMag")
        self._mag = value

        if self._mag is not None:
            self._mag.parent = self

    mag = property(
        fget=_get_mag,
        fset=_set_mag,
        doc=u"""Magnetic module output

        :Type: OutMag
        """,
    )

    def _get_struct(self):
        """getter of struct"""
        return self._struct

    def _set_struct(self, value):
        """setter of struct"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "struct"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutStruct()
        check_var("struct", value, "OutStruct")
        self._struct = value

        if self._struct is not None:
            self._struct.parent = self

    struct = property(
        fget=_get_struct,
        fset=_set_struct,
        doc=u"""Structural module output

        :Type: OutStruct
        """,
    )

    def _get_post(self):
        """getter of post"""
        return self._post

    def _set_post(self, value):
        """setter of post"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "post")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutPost()
        check_var("post", value, "OutPost")
        self._post = value

        if self._post is not None:
            self._post.parent = self

    post = property(
        fget=_get_post,
        fset=_set_post,
        doc=u"""Post-Processing settings

        :Type: OutPost
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_force(self):
        """getter of force"""
        return self._force

    def _set_force(self, value):
        """setter of force"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "force"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutForce()
        check_var("force", value, "OutForce")
        self._force = value

        if self._force is not None:
            self._force.parent = self

    force = property(
        fget=_get_force,
        fset=_set_force,
        doc=u"""Force module output

        :Type: OutForce
        """,
    )

    def _get_loss(self):
        """getter of loss"""
        return self._loss

    def _set_loss(self, value):
        """setter of loss"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "loss")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = OutLoss()
        check_var("loss", value, "OutLoss")
        self._loss = value

        if self._loss is not None:
            self._loss.parent = self

    loss = property(
        fget=_get_loss,
        fset=_set_loss,
        doc=u"""Loss module output

        :Type: OutLoss
        """,
    )



# -*-- coding: utf-8 -*


def translate(self, Zt):
    """Do the translation of the PolarArc

    Parameters
    ----------
    self : PolarArc
        a PolarArc object

    Zt : complex
        Complex value for translation

    Returns
    -------
    None
    """
    # check if the PolarArc is correct
    self.check()
    self.point_ref += Zt



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/SolutionData.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/SolutionData
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Solution import Solution

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.SolutionData.get_field import get_field
except ImportError as error:
    get_field = error

try:
    from ..Methods.Mesh.SolutionData.get_axes_list import get_axes_list
except ImportError as error:
    get_axes_list = error

try:
    from ..Methods.Mesh.SolutionData.get_solution import get_solution
except ImportError as error:
    get_solution = error


from ._check import InitUnKnowClassError


class SolutionData(Solution):
    """Define a Solution with SciDataTool objects."""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Mesh.SolutionData.get_field
    if isinstance(get_field, ImportError):
        get_field = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SolutionData method get_field: " + str(get_field)
                )
            )
        )
    else:
        get_field = get_field
    # cf Methods.Mesh.SolutionData.get_axes_list
    if isinstance(get_axes_list, ImportError):
        get_axes_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SolutionData method get_axes_list: " + str(get_axes_list)
                )
            )
        )
    else:
        get_axes_list = get_axes_list
    # cf Methods.Mesh.SolutionData.get_solution
    if isinstance(get_solution, ImportError):
        get_solution = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SolutionData method get_solution: " + str(get_solution)
                )
            )
        )
    else:
        get_solution = get_solution
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        field=None,
        type_cell="triangle",
        label=None,
        dimension=2,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "field" in list(init_dict.keys()):
                field = init_dict["field"]
            if "type_cell" in list(init_dict.keys()):
                type_cell = init_dict["type_cell"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
            if "dimension" in list(init_dict.keys()):
                dimension = init_dict["dimension"]
        # Set the properties (value check and convertion are done in setter)
        self.field = field
        # Call Solution init
        super(SolutionData, self).__init__(
            type_cell=type_cell, label=label, dimension=dimension
        )
        # The class is frozen (in Solution init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SolutionData_str = ""
        # Get the properties inherited from Solution
        SolutionData_str += super(SolutionData, self).__str__()
        SolutionData_str += "field = " + str(self.field) + linesep + linesep
        return SolutionData_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Solution
        if not super(SolutionData, self).__eq__(other):
            return False
        if other.field != self.field:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Solution
        diff_list.extend(super(SolutionData, self).compare(other, name=name))
        if (other.field is None and self.field is not None) or (
            other.field is not None and self.field is None
        ):
            diff_list.append(name + ".field None mismatch")
        elif self.field is not None:
            diff_list.extend(self.field.compare(other.field, name=name + ".field"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Solution
        S += super(SolutionData, self).__sizeof__()
        S += getsizeof(self.field)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Solution
        SolutionData_dict = super(SolutionData, self).as_dict(**kwargs)
        if self.field is None:
            SolutionData_dict["field"] = None
        else:
            SolutionData_dict["field"] = self.field.as_dict()
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SolutionData_dict["__class__"] = "SolutionData"
        return SolutionData_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.field = None
        # Set to None the properties inherited from Solution
        super(SolutionData, self)._set_None()

    def _get_field(self):
        """getter of field"""
        return self._field

    def _set_field(self, value):
        """setter of field"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "field"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("field", value, "DataND")
        self._field = value

    field = property(
        fget=_get_field,
        fset=_set_field,
        doc=u"""Data object containing the numerical values of a solution. One of the axis must be "Indices", a list of indices. If the solution is a vector, one of the axis must be "Direction", values ['x','y'] for example.

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )



# -*- coding: utf-8 -*-
from numpy import pi


def comp_length(self):
    """Compute the length of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Returns
    -------
    length: float
        length of the arc
    """

    R = self.comp_radius()  # Radius of the arc
    angle = self.angle

    # 2*pi*R is the length of the total circle
    # The arc is an alpha / 2pi portion of the cercle
    return float(2 * pi * R * (angle / (2 * pi)))
